@@ -1,15 +1,12 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:set ts=2 sw=2 sts=2 et: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-
 "use strict";
 
 // The widget module currently supports only Firefox.
 // See: https://bugzilla.mozilla.org/show_bug.cgi?id=560716
 module.metadata = {
-  "stability": "stable",
+  "stability": "deprecated",
   "engines": {
     "Firefox": "*"
   }
@@ -28,6 +25,8 @@ const ERR_CONTENT = "No content or contentURL property found. Widgets must "
                "position.",
       ERR_DESTROYED = "The widget has been destroyed and can no longer be used.";
 
+const INSERTION_PREF_ROOT = "extensions.sdk-widget-inserted.";
+
 // Supported events, mapping from DOM event names to our event names
 const EVENTS = {
   "click": "click",
@@ -35,20 +34,32 @@ const EVENTS = {
   "mouseout": "mouseout",
 };
 
+// In the Australis menu panel, normally widgets should be treated like
+// normal toolbarbuttons. If they're any wider than this margin, we'll
+// treat them as wide widgets instead, which fill up the width of the panel:
+const AUSTRALIS_PANEL_WIDE_WIDGET_CUTOFF = 70;
+const AUSTRALIS_PANEL_WIDE_CLASSNAME = "panel-wide-item";
+
 const { validateOptions } = require("./deprecated/api-utils");
 const panels = require("./panel");
 const { EventEmitter, EventEmitterTrait } = require("./deprecated/events");
 const { Trait } = require("./deprecated/traits");
 const LightTrait = require('./deprecated/light-traits').Trait;
-const { Loader, Symbiont } = require("./content/content");
+const { Loader } = require("./content/loader");
+const { Symbiont } = require("./deprecated/symbiont");
 const { Cortex } = require('./deprecated/cortex');
 const windowsAPI = require("./windows");
 const { WindowTracker } = require("./deprecated/window-utils");
 const { isBrowser } = require("./window/utils");
 const { setTimeout } = require("./timers");
 const unload = require("./system/unload");
-const { uuid } = require("./util/uuid");
 const { getNodeView } = require("./view/core");
+const prefs = require('./preferences/service');
+
+require("./util/deprecate").deprecateUsage(
+  "The widget module is deprecated.  " +
+  "Please consider using the sdk/ui module instead."
+);
 
 // Data types definition
 const valid = {
@@ -217,6 +228,15 @@ let model = {
 
 };
 
+function saveInserted(widgetId) {
+  prefs.set(INSERTION_PREF_ROOT + widgetId, true);
+}
+
+function haveInserted(widgetId) {
+  return prefs.has(INSERTION_PREF_ROOT + widgetId);
+}
+
+const isWide = node => node.classList.contains(AUSTRALIS_PANEL_WIDE_CLASSNAME);
 
 /**
  * Main Widget class: entry point of the widget API
@@ -431,11 +451,12 @@ const WidgetViewTrait = LightTrait.compose(EventEmitterTrait, LightTrait({
 
     // Special case for click events: if the widget doesn't have a click
     // handler, but it does have a panel, display the panel.
-    if ("click" == type && !this._listeners("click").length && this.panel)
+    if ("click" == type && !this._listeners("click").length && this.panel) {
       // This kind of ugly workaround, instead we should implement
       // `getNodeView` for the `Widget` class itself, but that's kind of
       // hard without cleaning things up.
-      this.panel.show(getNodeView.implement({}, function() domNode));
+      this.panel.show(null, getNodeView.implement({}, () => domNode));
+    }
   },
 
   _isInWindow: function WidgetView__isInWindow(window) {
@@ -557,6 +578,9 @@ let browserManager = {
     let idx = this.items.indexOf(item);
     if (idx > -1)
       this.items.splice(idx, 1);
+  },
+  propagateCurrentset: function browserManager_propagateCurrentset(id, currentset) {
+    this.windows.forEach(function (w) w.doc.getElementById(id).setAttribute("currentset", currentset));
   }
 };
 
@@ -604,56 +628,21 @@ BrowserWindow.prototype = {
     let palette = toolbox.palette;
     palette.appendChild(node);
 
-    // Search for widget toolbar by reading toolbar's currentset attribute
-    let container = null;
-    let toolbars = this.doc.getElementsByTagName("toolbar");
-    let id = node.getAttribute("id");
-    for (let i = 0, l = toolbars.length; i < l; i++) {
-      let toolbar = toolbars[i];
-      if (toolbar.getAttribute("currentset").indexOf(id) == -1)
-        continue;
-      container = toolbar;
+    let { CustomizableUI } = this.window;
+    let { id } = node;
+
+    let placement = CustomizableUI.getPlacementOfWidget(id);
+
+    if (!placement) {
+      if (haveInserted(id) || isWide(node))
+        return;
+
+      placement = {area: 'nav-bar', position: undefined};
+      saveInserted(id);
     }
 
-    // if widget isn't in any toolbar, add it to the addon-bar
-    // TODO: we may want some "first-launch" module to do this only on very
-    // first execution
-    if (!container) {
-      container = this.doc.getElementById("addon-bar");
-      // TODO: find a way to make the following code work when we use "cfx run":
-      // http://mxr.mozilla.org/mozilla-central/source/browser/base/content/browser.js#8586
-      // until then, force display of addon bar directly from sdk code
-      // https://bugzilla.mozilla.org/show_bug.cgi?id=627484
-      if (container.collapsed)
-        this.window.toggleAddonBar();
-    }
-
-    // Now retrieve a reference to the next toolbar item
-    // by reading currentset attribute on the toolbar
-    let nextNode = null;
-    let currentSet = container.getAttribute("currentset");
-    let ids = (currentSet == "__empty") ? [] : currentSet.split(",");
-    let idx = ids.indexOf(id);
-    if (idx != -1) {
-      for (let i = idx; i < ids.length; i++) {
-        nextNode = this.doc.getElementById(ids[i]);
-        if (nextNode)
-          break;
-      }
-    }
-
-    // Finally insert our widget in the right toolbar and in the right position
-    container.insertItem(id, nextNode, null, false);
-
-    // Update DOM in order to save position: which toolbar, and which position
-    // in this toolbar. But only do this the first time we add it to the toolbar
-    // Otherwise, this code will collide with other instance of Widget module
-    // during Firefox startup. See bug 685929.
-    if (ids.indexOf(id) == -1) {
-      container.setAttribute("currentset", container.currentSet);
-      // Save DOM attribute in order to save position on new window opened
-      this.window.document.persist(container.id, "currentset");
-    }
+    CustomizableUI.addWidgetToArea(id, placement.area, placement.position);
+    CustomizableUI.ensureWidgetPlacedInWindow(id, this.window);
   }
 }
 
@@ -702,7 +691,6 @@ WidgetChrome.prototype.update = function WC_update(updatedItem, property, value)
 WidgetChrome.prototype._createNode = function WC__createNode() {
   // XUL element container for widget
   let node = this._doc.createElement("toolbaritem");
-  let guid = String(uuid());
 
   // Temporary work around require("self") failing on unit-test execution ...
   let jetpackID = "testID";
@@ -719,6 +707,13 @@ WidgetChrome.prototype._createNode = function WC__createNode() {
   // Bug 626326: Prevent customize toolbar context menu to appear
   node.setAttribute("context", "");
 
+  // For use in styling by the browser
+  node.setAttribute("sdkstylewidget", "true");
+
+  if (this._widget.width > AUSTRALIS_PANEL_WIDE_WIDGET_CUTOFF) {
+    node.classList.add(AUSTRALIS_PANEL_WIDE_CLASSNAME);
+  }
+
   // TODO move into a stylesheet, configurable by consumers.
   // Either widget.style, exposing the style object, or a URL
   // (eg, can load local stylesheet file).
@@ -734,8 +729,10 @@ WidgetChrome.prototype._createNode = function WC__createNode() {
 
 // Initial population of a widget's content.
 WidgetChrome.prototype.fill = function WC_fill() {
+  let { node, _doc: document } = this;
+
   // Create element
-  var iframe = this._doc.createElement("iframe");
+  let iframe = document.createElement("iframe");
   iframe.setAttribute("type", "content");
   iframe.setAttribute("transparent", "transparent");
   iframe.style.overflow = "hidden";
@@ -748,7 +745,23 @@ WidgetChrome.prototype.fill = function WC_fill() {
 
   // Do this early, because things like contentWindow are null
   // until the node is attached to a document.
-  this.node.appendChild(iframe);
+  node.appendChild(iframe);
+
+  let label = document.createElement("label");
+  label.setAttribute("value", this._widget.label);
+  label.className = "toolbarbutton-text";
+  label.setAttribute("crop", "right");
+  label.setAttribute("flex", "1");
+  node.appendChild(label);
+
+  // This toolbarbutton is created to provide a more consistent user experience
+  // during customization, see:
+  // https://bugzilla.mozilla.org/show_bug.cgi?id=959640
+  let button = document.createElement("toolbarbutton");
+  button.setAttribute("label", this._widget.label);
+  button.setAttribute("crop", "right");
+  button.className = "toolbarbutton-1 chromeclass-toolbar-additional";
+  node.appendChild(button);
 
   // add event handlers
   this.addEventHandlers();
