@@ -6,20 +6,87 @@
 const cssAutoCompleter = require("devtools/sourceeditor/css-autocompleter");
 const { AutocompletePopup } = require("devtools/shared/autocomplete-popup");
 
+const CM_TERN_SCRIPTS = [
+  "chrome://browser/content/devtools/codemirror/tern.js",
+  "chrome://browser/content/devtools/codemirror/show-hint.js"
+];
+
 const privates = new WeakMap();
 
 /**
- * Prepares an editor instance for autocompletion, setting up the popup and the
- * CSS completer instance.
+ * Prepares an editor instance for autocompletion.
  */
-function setupAutoCompletion(ctx, walker) {
+function setupAutoCompletion(ctx, options) {
   let { cm, ed, Editor } = ctx;
 
   let win = ed.container.contentWindow.wrappedJSObject;
+  let { CodeMirror, document } = win;
 
   let completer = null;
-  if (ed.config.mode == Editor.modes.css)
-    completer = new cssAutoCompleter({walker: walker});
+  let autocompleteKey = "Ctrl-" +
+                        Editor.keyFor("autocompletion", { noaccel: true });
+  if (ed.config.mode == Editor.modes.js) {
+    let defs = [
+      "tern/browser",
+      "tern/ecma5",
+    ].map(require);
+
+    CM_TERN_SCRIPTS.forEach(ed.loadScript, ed);
+    win.tern = require("tern/tern");
+    cm.tern = new CodeMirror.TernServer({
+      defs: defs,
+      typeTip: function(data) {
+        let tip = document.createElement("span");
+        tip.className = "CodeMirror-Tern-information";
+        let tipType = document.createElement("strong");
+        tipType.appendChild(document.createTextNode(data.type || "not found"));
+        tip.appendChild(tipType);
+
+        if (data.doc) {
+          tip.appendChild(document.createTextNode(" — " + data.doc));
+        }
+
+        if (data.url) {
+          tip.appendChild(document.createTextNode(" "));
+          let docLink = document.createElement("a");
+          docLink.textContent = "[docs]";
+          docLink.href = data.url;
+          docLink.className = "theme-link";
+          docLink.setAttribute("target", "_blank");
+          tip.appendChild(docLink);
+        }
+
+        return tip;
+      }
+    });
+    cm.on("cursorActivity", (cm) => {
+      cm.tern.updateArgHints(cm);
+    });
+
+    let keyMap = {};
+
+    keyMap[autocompleteKey] = (cm) => {
+      cm.tern.getHint(cm, (data) => {
+        CodeMirror.on(data, "shown", () => ed.emit("before-suggest"));
+        CodeMirror.on(data, "close", () => ed.emit("after-suggest"));
+        CodeMirror.on(data, "select", () => ed.emit("suggestion-entered"));
+        CodeMirror.showHint(cm, (cm, cb) => cb(data), { async: true });
+      });
+    };
+
+    keyMap[Editor.keyFor("showInformation", { noaccel: true })] = (cm) => {
+      cm.tern.showType(cm, null, () => {
+        ed.emit("show-information");
+      });
+    };
+
+    cm.addKeyMap(keyMap);
+
+    // TODO: Integrate tern autocompletion with this autocomplete API.
+    return;
+  } else if (ed.config.mode == Editor.modes.css) {
+    completer = new cssAutoCompleter({walker: options.walker});
+  }
 
   let popup = new AutocompletePopup(win.parent.document, {
     position: "after_start",
@@ -34,30 +101,32 @@ function setupAutoCompletion(ctx, walker) {
       return;
     }
 
-    return win.CodeMirror.Pass;
+    return CodeMirror.Pass;
   };
 
   let keyMap = {
     "Tab": cycle,
     "Down": cycle,
-    "Shift-Tab": cycle.bind(this, true),
-    "Up": cycle.bind(this, true),
+    "Shift-Tab": cycle.bind(null, true),
+    "Up": cycle.bind(null, true),
     "Enter": () => {
       if (popup && popup.isOpen) {
         if (!privates.get(ed).suggestionInsertedOnce) {
           privates.get(ed).insertingSuggestion = true;
-          let {label, preLabel} = popup.getItemAtIndex(0);
+          let {label, preLabel, text} = popup.getItemAtIndex(0);
           let cur = ed.getCursor();
-          ed.replaceText(label.slice(preLabel.length), cur, cur);
+          ed.replaceText(text.slice(preLabel.length), cur, cur);
         }
         popup.hidePopup();
+        // This event is used in tests
+        ed.emit("popup-hidden");
         return;
       }
 
-      return win.CodeMirror.Pass;
+      return CodeMirror.Pass;
     }
   };
-  keyMap[Editor.accel("Space")] = cm => autoComplete(ctx);
+  keyMap[autocompleteKey] = cm => autoComplete(ctx);
   cm.addKeyMap(keyMap);
 
   cm.on("keydown", (cm, e) => onEditorKeypress(ctx, e));
@@ -91,7 +160,7 @@ function autoComplete({ ed, cm }) {
   let cur = ed.getCursor();
   completer.complete(cm.getRange({line: 0, ch: 0}, cur), cur)
     .then(suggestions => {
-    if (!suggestions || !suggestions.length || !suggestions[0].preLabel) {
+    if (!suggestions || !suggestions.length || suggestions[0].preLabel == null) {
       private.suggestionInsertedOnce = false;
       popup.hidePopup();
       ed.emit("after-suggest");
@@ -101,10 +170,12 @@ function autoComplete({ ed, cm }) {
     // "backgr|" but we need to open the popup at the beginning of the character
     // "b". Thus we need to calculate the width of the entered part of the token
     // ("backgr" here). 4 comes from the popup's left padding.
+
+    let cursorElement = cm.display.cursorDiv.querySelector(".CodeMirror-cursor");
     let left = suggestions[0].preLabel.length * cm.defaultCharWidth() + 4;
     popup.hidePopup();
     popup.setItems(suggestions);
-    popup.openPopup(cm.display.cursor, -1 * left, 0);
+    popup.openPopup(cursorElement, -1 * left, 0);
     private.suggestionInsertedOnce = false;
     // This event is used in tests.
     ed.emit("after-suggest");
@@ -135,17 +206,17 @@ function cycleSuggestions(ed, reverse) {
     }
     if (popup.itemCount == 1)
       popup.hidePopup();
-    ed.replaceText(firstItem.label.slice(firstItem.preLabel.length), cur, cur);
+    ed.replaceText(firstItem.text.slice(firstItem.preLabel.length), cur, cur);
   } else {
     let fromCur = {
       line: cur.line,
-      ch  : cur.ch - popup.selectedItem.label.length
+      ch  : cur.ch - popup.selectedItem.text.length
     };
     if (reverse)
       popup.selectPreviousItem();
     else
       popup.selectNextItem();
-    ed.replaceText(popup.selectedItem.label, fromCur, cur);
+    ed.replaceText(popup.selectedItem.text, fromCur, cur);
   }
   // This event is used in tests.
   ed.emit("suggestion-entered");
@@ -157,7 +228,33 @@ function cycleSuggestions(ed, reverse) {
  */
 function onEditorKeypress({ ed, Editor }, event) {
   let private = privates.get(ed);
+
+  // Do not try to autocomplete with multiple selections.
+  if (ed.hasMultipleSelections()) {
+    private.doNotAutocomplete = true;
+    private.popup.hidePopup();
+    return;
+  }
+
+  if ((event.ctrlKey || event.metaKey) && event.keyCode == event.DOM_VK_SPACE) {
+    // When Ctrl/Cmd + Space is pressed, two simultaneous keypresses are emitted
+    // first one for just the Ctrl/Cmd and second one for combo. The first one
+    // leave the private.doNotAutocomplete as true, so we have to make it false
+    private.doNotAutocomplete = false;
+    return;
+  }
+
+  if (event.ctrlKey || event.metaKey || event.altKey) {
+    private.doNotAutocomplete = true;
+    private.popup.hidePopup();
+    return;
+  }
+
   switch (event.keyCode) {
+    case event.DOM_VK_RETURN:
+      private.doNotAutocomplete = true;
+      break;
+
     case event.DOM_VK_ESCAPE:
       if (private.popup.isOpen)
         event.preventDefault();
@@ -188,7 +285,21 @@ function onEditorKeypress({ ed, Editor }, event) {
 function getPopup({ ed }) {
   return privates.get(ed).popup;
 }
+
+/**
+ * Returns contextual information about the token covered by the caret if the
+ * implementation of completer supports it.
+ */
+function getInfoAt({ ed }, caret) {
+  let completer = privates.get(ed).completer;
+  if (completer && completer.getInfoAt)
+    return completer.getInfoAt(ed.getText(), caret);
+
+  return null;
+}
+
 // Export functions
 
 module.exports.setupAutoCompletion = setupAutoCompletion;
 module.exports.getAutocompletionPopup = getPopup;
+module.exports.getInfoAt = getInfoAt;
