@@ -1,4 +1,4 @@
-/* -*- Mode: js2; js2-basic-offset: 2; indent-tabs-mode: nil; -*- */
+/* -*- js2-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,9 +11,9 @@ const {Cc, Ci, Cu} = require("chrome");
 let WebConsoleUtils = require("devtools/toolkit/webconsole/utils").Utils;
 let Heritage = require("sdk/core/heritage");
 
-loader.lazyGetter(this, "promise", () => require("sdk/core/promise"));
 loader.lazyGetter(this, "Telemetry", () => require("devtools/shared/telemetry"));
 loader.lazyGetter(this, "WebConsoleFrame", () => require("devtools/webconsole/webconsole").WebConsoleFrame);
+loader.lazyImporter(this, "promise", "resource://gre/modules/Promise.jsm", "Promise");
 loader.lazyImporter(this, "gDevTools", "resource:///modules/devtools/gDevTools.jsm");
 loader.lazyImporter(this, "devtools", "resource://gre/modules/devtools/Loader.jsm");
 loader.lazyImporter(this, "Services", "resource://gre/modules/Services.jsm");
@@ -28,6 +28,8 @@ const BROWSER_CONSOLE_WINDOW_FEATURES = "chrome,titlebar,toolbar,centerscreen,re
 // The preference prefix for all of the Browser Console filters.
 const BROWSER_CONSOLE_FILTER_PREFS_PREFIX = "devtools.browserconsole.filter.";
 
+let gHudId = 0;
+
 ///////////////////////////////////////////////////////////////////////////
 //// The HUD service
 
@@ -108,6 +110,7 @@ HUD_SERVICE.prototype =
   function HS_openBrowserConsole(aTarget, aIframeWindow, aChromeWindow)
   {
     let hud = new BrowserConsole(aTarget, aIframeWindow, aChromeWindow);
+    this._browserConsoleID = hud.hudId;
     this.consoles.set(hud.hudId, hud);
     return hud.init();
   },
@@ -140,24 +143,6 @@ HUD_SERVICE.prototype =
     return this.consoles.get(aId);
   },
 
-  /**
-   * Toggle the Web Console for the current tab.
-   *
-   * @return object
-   *         A promise for either the opening of the toolbox that holds the Web
-   *         Console, or a Promise for the closing of the toolbox.
-   */
-  toggleWebConsole: function HS_toggleWebConsole()
-  {
-    let window = this.currentContext();
-    let target = devtools.TargetFactory.forTab(window.gBrowser.selectedTab);
-    let toolbox = gDevTools.getToolbox(target);
-
-    return toolbox && toolbox.currentToolId == "webconsole" ?
-        toolbox.destroy() :
-        gDevTools.showToolbox(target, "webconsole");
-  },
-
   /**
    * Find if there is a Web Console open for the current tab and return the
    * instance.
@@ -206,7 +191,7 @@ HUD_SERVICE.prototype =
       client.connect(() =>
         client.listTabs((aResponse) => {
           // Add Global Process debugging...
-          let globals = JSON.parse(JSON.stringify(aResponse));
+          let globals = Cu.cloneInto(aResponse, {});
           delete globals.tabs;
           delete globals.selected;
           // ...only if there are appropriate actors (a 'from' property will
@@ -254,13 +239,13 @@ HUD_SERVICE.prototype =
       return deferred.promise;
     }
 
-    connect().then(getTarget).then(openWindow).then((aWindow) =>
+    connect().then(getTarget).then(openWindow).then((aWindow) => {
       this.openBrowserConsole(target, aWindow, aWindow)
         .then((aBrowserConsole) => {
-          this._browserConsoleID = aBrowserConsole.hudId;
           this._browserConsoleDefer.resolve(aBrowserConsole);
           this._browserConsoleDefer = null;
-        }));
+        })
+    }, console.error);
 
     return this._browserConsoleDefer.promise;
   },
@@ -300,7 +285,7 @@ function WebConsole(aTarget, aIframeWindow, aChromeWindow)
 {
   this.iframeWindow = aIframeWindow;
   this.chromeWindow = aChromeWindow;
-  this.hudId = "hud_" + Date.now();
+  this.hudId = "hud_" + ++gHudId;
   this.target = aTarget;
 
   this.browserWindow = this.chromeWindow.top;
@@ -332,13 +317,29 @@ WebConsole.prototype = {
    */
   get lastFinishedRequestCallback() HUDService.lastFinishedRequest.callback,
 
+  /**
+   * Getter for the window that can provide various utilities that the web
+   * console makes use of, like opening links, managing popups, etc.  In
+   * most cases, this will be |this.browserWindow|, but in some uses (such as
+   * the Browser Toolbox), there is no browser window, so an alternative window
+   * hosts the utilities there.
+   * @type nsIDOMWindow
+   */
+  get chromeUtilsWindow()
+  {
+    if (this.browserWindow) {
+      return this.browserWindow;
+    }
+    return this.chromeWindow.top;
+  },
+
   /**
    * Getter for the xul:popupset that holds any popups we open.
    * @type nsIDOMElement
    */
   get mainPopupSet()
   {
-    return this.browserWindow.document.getElementById("mainPopupSet");
+    return this.chromeUtilsWindow.document.getElementById("mainPopupSet");
   },
 
   /**
@@ -350,7 +351,10 @@ WebConsole.prototype = {
     return this.ui ? this.ui.outputNode : null;
   },
 
-  get gViewSourceUtils() this.browserWindow.gViewSourceUtils,
+  get gViewSourceUtils()
+  {
+    return this.chromeUtilsWindow.gViewSourceUtils;
+  },
 
   /**
    * Initialize the Web Console instance.
@@ -413,7 +417,7 @@ WebConsole.prototype = {
    */
   openLink: function WC_openLink(aLink)
   {
-    this.browserWindow.openUILinkIn(aLink, "tab");
+    this.chromeUtilsWindow.openUILinkIn(aLink, "tab");
   },
 
   /**
@@ -474,55 +478,71 @@ WebConsole.prototype = {
   viewSourceInDebugger:
   function WC_viewSourceInDebugger(aSourceURL, aSourceLine)
   {
-    let self = this;
-    let panelWin = null;
-    let debuggerWasOpen = true;
     let toolbox = gDevTools.getToolbox(this.target);
     if (!toolbox) {
-      self.viewSource(aSourceURL, aSourceLine);
+      this.viewSource(aSourceURL, aSourceLine);
       return;
     }
 
-    if (!toolbox.getPanel("jsdebugger")) {
-      debuggerWasOpen = false;
-      let toolboxWin = toolbox.doc.defaultView;
-      toolboxWin.addEventListener("Debugger:AfterSourcesAdded",
-                                  function afterSourcesAdded() {
-        toolboxWin.removeEventListener("Debugger:AfterSourcesAdded",
-                                       afterSourcesAdded);
-        loadScript();
-      });
+    let showSource = ({ DebuggerView }) => {
+      if (DebuggerView.Sources.containsValue(aSourceURL)) {
+        DebuggerView.setEditorLocation(aSourceURL, aSourceLine,
+                                       { noDebug: true }).then(() => {
+          this.ui.emit("source-in-debugger-opened");
+        });
+        return;
+      }
+      toolbox.selectTool("webconsole");
+      this.viewSource(aSourceURL, aSourceLine);
     }
 
-    toolbox.selectTool("jsdebugger").then(function onDebuggerOpen(dbg) {
-      panelWin = dbg.panelWin;
-      if (debuggerWasOpen) {
-        loadScript();
+    // If the Debugger was already open, switch to it and try to show the
+    // source immediately. Otherwise, initialize it and wait for the sources
+    // to be added first.
+    let debuggerAlreadyOpen = toolbox.getPanel("jsdebugger");
+    toolbox.selectTool("jsdebugger").then(({ panelWin: dbg }) => {
+      if (debuggerAlreadyOpen) {
+        showSource(dbg);
+      } else {
+        dbg.once(dbg.EVENTS.SOURCES_ADDED, () => showSource(dbg));
       }
     });
+  },
 
-    function loadScript() {
-      let debuggerView = panelWin.DebuggerView;
-      if (!debuggerView.Sources.containsValue(aSourceURL)) {
-        toolbox.selectTool("webconsole");
-        self.viewSource(aSourceURL, aSourceLine);
+
+  /**
+   * Tries to open a JavaScript file related to the web page for the web console
+   * instance in the corresponding Scratchpad.
+   *
+   * @param string aSourceURL
+   *        The URL of the file which corresponds to a Scratchpad id.
+   */
+  viewSourceInScratchpad: function WC_viewSourceInScratchpad(aSourceURL)
+  {
+    // Check for matching top level Scratchpad window.
+    let wins = Services.wm.getEnumerator("devtools:scratchpad");
+
+    while (wins.hasMoreElements()) {
+      let win = wins.getNext();
+
+      if (!win.closed && win.Scratchpad.uniqueName === aSourceURL) {
+        win.focus();
         return;
       }
-      if (debuggerWasOpen && debuggerView.Sources.selectedValue == aSourceURL) {
-        debuggerView.editor.setCaretPosition(aSourceLine - 1);
-        return;
-      }
-
-      panelWin.addEventListener("Debugger:SourceShown", onSource, false);
-      debuggerView.Sources.preferredSource = aSourceURL;
     }
 
-    function onSource(aEvent) {
-      if (aEvent.detail.url != aSourceURL) {
-        return;
+    // Check for matching Scratchpad toolbox tab.
+    for (let [, toolbox] of gDevTools) {
+      let scratchpadPanel = toolbox.getPanel("scratchpad");
+      if (scratchpadPanel) {
+        let { scratchpad } = scratchpadPanel;
+        if (scratchpad.uniqueName === aSourceURL) {
+          toolbox.selectTool("scratchpad");
+          toolbox.raise();
+          scratchpad.editor.focus();
+          return;
+        }
       }
-      panelWin.removeEventListener("Debugger:SourceShown", onSource, false);
-      panelWin.DebuggerView.editor.setCaretPosition(aSourceLine - 1);
     }
   },
 
@@ -549,12 +569,12 @@ WebConsole.prototype = {
     if (!panel) {
       return null;
     }
-    let framesController = panel.panelWin.gStackFrames;
+    let framesController = panel.panelWin.DebuggerController.StackFrames;
     let thread = framesController.activeThread;
     if (thread && thread.paused) {
       return {
         frames: thread.cachedFrames,
-        selected: framesController.currentFrame,
+        selected: framesController.currentFrameDepth,
       };
     }
     return null;
@@ -706,7 +726,7 @@ BrowserConsole.prototype = Heritage.extend(WebConsole.prototype,
 const HUDService = new HUD_SERVICE();
 
 (() => {
-  let methods = ["openWebConsole", "openBrowserConsole", "toggleWebConsole",
+  let methods = ["openWebConsole", "openBrowserConsole",
                  "toggleBrowserConsole", "getOpenWebConsole",
                  "getBrowserConsole", "getHudByWindow", "getHudReferenceById"];
   for (let method of methods) {
