@@ -64,9 +64,18 @@ public:
     SetHTMLAttr(nsGkAtoms::media, aMedia, rv);
   }
 
+  void GetKeySystem(nsString& aKeySystem) const
+  {
+    GetHTMLAttr(nsGkAtoms::keysystem, aKeySystem);
+  }
+
+  void SetKeySystem(const nsAString& aKeySystem)
+  {
+    SetHTMLAttr(nsGkAtoms::keysystem, aKeySystem);
+  }
+
 protected:
-  virtual JSObject* WrapNode(JSContext* aCx,
-                             JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;
+  virtual JSObject* WrapNode(JSContext* aCx) MOZ_OVERRIDE;
 
 protected:
   virtual void GetItemValueText(nsAString& text) MOZ_OVERRIDE;
