@@ -14,7 +14,6 @@
 
 #include "nsXULContentSink.h"
 
-#include "jsapi.h"
 #include "jsfriendapi.h"
 
 #include "nsCOMPtr.h"
@@ -26,13 +25,12 @@
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMXULDocument.h"
 #include "nsIFormControl.h"
-#include "nsINameSpaceManager.h"
 #include "nsINodeInfo.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
-#include "nsIScriptRuntime.h"
 #include "nsIServiceManager.h"
 #include "nsIURL.h"
+#include "nsNameSpaceManager.h"
 #include "nsParserBase.h"
 #include "nsViewManager.h"
 #include "nsIXULDocument.h"
@@ -57,9 +55,10 @@
 #include "nsXMLContentSink.h"
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
+#include "nsContentTypeParser.h"
 
 #ifdef PR_LOGGING
-static PRLogModuleInfo* gLog;
+static PRLogModuleInfo* gContentSinkLog;
 #endif
 
 //----------------------------------------------------------------------
@@ -170,8 +169,8 @@ XULContentSinkImpl::XULContentSinkImpl()
 {
 
 #ifdef PR_LOGGING
-    if (! gLog)
-        gLog = PR_NewLogModule("nsXULContentSink");
+    if (! gContentSinkLog)
+        gContentSinkLog = PR_NewLogModule("nsXULContentSink");
 #endif
 }
 
@@ -190,6 +189,8 @@ XULContentSinkImpl::~XULContentSinkImpl()
 //----------------------------------------------------------------------
 // nsISupports interface
 
+NS_IMPL_CYCLE_COLLECTION_CLASS(XULContentSinkImpl)
+
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(XULContentSinkImpl)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mNodeInfoManager)
   tmp->mContextStack.Clear();
@@ -335,7 +336,7 @@ XULContentSinkImpl::Init(nsIDocument* aDocument,
 //
 
 bool
-XULContentSinkImpl::IsDataInBuffer(PRUnichar* buffer, int32_t length)
+XULContentSinkImpl::IsDataInBuffer(char16_t* buffer, int32_t length)
 {
     for (int32_t i = 0; i < length; ++i) {
         if (buffer[i] == ' ' ||
@@ -411,7 +412,7 @@ XULContentSinkImpl::FlushText(bool aCreateTextNode)
 //----------------------------------------------------------------------
 
 nsresult
-XULContentSinkImpl::NormalizeAttributeString(const PRUnichar *aExpatName,
+XULContentSinkImpl::NormalizeAttributeString(const char16_t *aExpatName,
                                              nsAttrName &aName)
 {
     int32_t nameSpaceID;
@@ -452,16 +453,14 @@ XULContentSinkImpl::CreateElement(nsINodeInfo *aNodeInfo,
 
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleStartElement(const PRUnichar *aName, 
-                                       const PRUnichar **aAtts,
+XULContentSinkImpl::HandleStartElement(const char16_t *aName, 
+                                       const char16_t **aAtts,
                                        uint32_t aAttsCount, 
-                                       int32_t aIndex, 
                                        uint32_t aLineNumber)
 { 
   // XXX Hopefully the parser will flag this before we get here. If
   // we're in the epilog, there should be no new elements
   NS_PRECONDITION(mState != eInEpilog, "tag in XUL doc epilog");
-  NS_PRECONDITION(aIndex >= -1, "Bogus aIndex");
   NS_PRECONDITION(aAttsCount % 2 == 0, "incorrect aAttsCount");
   // Adjust aAttsCount so it's the actual number of attributes
   aAttsCount /= 2;
@@ -495,27 +494,18 @@ XULContentSinkImpl::HandleStartElement(const PRUnichar *aName,
 
   case eInEpilog:
   case eInScript:
-      PR_LOG(gLog, PR_LOG_WARNING,
+      PR_LOG(gContentSinkLog, PR_LOG_WARNING,
              ("xul: warning: unexpected tags in epilog at line %d",
              aLineNumber));
       rv = NS_ERROR_UNEXPECTED; // XXX
       break;
   }
 
-  // Set the ID attribute atom on the node info object for this node
-  if (aIndex != -1 && NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsIAtom> IDAttr = do_GetAtom(aAtts[aIndex]);
-
-    if (IDAttr) {
-      nodeInfo->SetIDAttributeAtom(IDAttr);
-    }
-  }
-
   return rv;
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleEndElement(const PRUnichar *aName)
+XULContentSinkImpl::HandleEndElement(const char16_t *aName)
 {
     // Never EVER return anything but NS_OK or
     // NS_ERROR_HTMLPARSER_BLOCK from this method. Doing so will blow
@@ -565,7 +555,7 @@ XULContentSinkImpl::HandleEndElement(const PRUnichar *aName)
             script->mOutOfLine = false;
             if (doc)
                 script->Compile(mText, mTextLength, mDocumentURL,
-                                script->mLineNo, doc, mPrototype);
+                                script->mLineNo, doc);
         }
 
         FlushText(false);
@@ -602,14 +592,14 @@ XULContentSinkImpl::HandleEndElement(const PRUnichar *aName)
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleComment(const PRUnichar *aName)
+XULContentSinkImpl::HandleComment(const char16_t *aName)
 {
    FlushText();
    return NS_OK;
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleCDataSection(const PRUnichar *aData, uint32_t aLength)
+XULContentSinkImpl::HandleCDataSection(const char16_t *aData, uint32_t aLength)
 {
     FlushText();
     return AddText(aData, aLength);
@@ -626,7 +616,7 @@ XULContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset,
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleCharacterData(const PRUnichar *aData, 
+XULContentSinkImpl::HandleCharacterData(const char16_t *aData, 
                                         uint32_t aLength)
 {
   if (aData && mState != eInProlog && mState != eInEpilog) {
@@ -636,8 +626,8 @@ XULContentSinkImpl::HandleCharacterData(const PRUnichar *aData,
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget, 
-                                                const PRUnichar *aData)
+XULContentSinkImpl::HandleProcessingInstruction(const char16_t *aTarget, 
+                                                const char16_t *aData)
 {
     FlushText();
 
@@ -673,8 +663,8 @@ XULContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
 
 
 NS_IMETHODIMP
-XULContentSinkImpl::HandleXMLDeclaration(const PRUnichar *aVersion,
-                                         const PRUnichar *aEncoding,
+XULContentSinkImpl::HandleXMLDeclaration(const char16_t *aVersion,
+                                         const char16_t *aEncoding,
                                          int32_t aStandalone)
 {
   return NS_OK;
@@ -682,8 +672,8 @@ XULContentSinkImpl::HandleXMLDeclaration(const PRUnichar *aVersion,
 
 
 NS_IMETHODIMP
-XULContentSinkImpl::ReportError(const PRUnichar* aErrorText, 
-                                const PRUnichar* aSourceText,
+XULContentSinkImpl::ReportError(const char16_t* aErrorText, 
+                                const char16_t* aSourceText,
                                 nsIScriptError *aError,
                                 bool *_retval)
 {
@@ -711,26 +701,26 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
     return NS_OK;
   }
 
-  const PRUnichar* noAtts[] = { 0, 0 };
+  const char16_t* noAtts[] = { 0, 0 };
 
   NS_NAMED_LITERAL_STRING(errorNs,
                           "http://www.mozilla.org/newlayout/xml/parsererror.xml");
 
   nsAutoString parsererror(errorNs);
-  parsererror.Append((PRUnichar)0xFFFF);
+  parsererror.Append((char16_t)0xFFFF);
   parsererror.AppendLiteral("parsererror");
   
-  rv = HandleStartElement(parsererror.get(), noAtts, 0, -1, 0);
+  rv = HandleStartElement(parsererror.get(), noAtts, 0, 0);
   NS_ENSURE_SUCCESS(rv,rv);
 
   rv = HandleCharacterData(aErrorText, NS_strlen(aErrorText));
   NS_ENSURE_SUCCESS(rv,rv);  
   
   nsAutoString sourcetext(errorNs);
-  sourcetext.Append((PRUnichar)0xFFFF);
+  sourcetext.Append((char16_t)0xFFFF);
   sourcetext.AppendLiteral("sourcetext");
 
-  rv = HandleStartElement(sourcetext.get(), noAtts, 0, -1, 0);
+  rv = HandleStartElement(sourcetext.get(), noAtts, 0, 0);
   NS_ENSURE_SUCCESS(rv,rv);
   
   rv = HandleCharacterData(aSourceText, NS_strlen(aSourceText));
@@ -746,7 +736,7 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
 }
 
 nsresult
-XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes, 
+XULContentSinkImpl::OpenRoot(const char16_t** aAttributes, 
                              const uint32_t aAttrLen, 
                              nsINodeInfo *aNodeInfo)
 {
@@ -758,7 +748,7 @@ XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes,
 
     if (aNodeInfo->Equals(nsGkAtoms::script, kNameSpaceID_XHTML) || 
         aNodeInfo->Equals(nsGkAtoms::script, kNameSpaceID_XUL)) {
-        PR_LOG(gLog, PR_LOG_ERROR,
+        PR_LOG(gContentSinkLog, PR_LOG_ERROR,
                ("xul: script tag not allowed as root content element"));
 
         return NS_ERROR_UNEXPECTED;
@@ -770,10 +760,10 @@ XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes,
 
     if (NS_FAILED(rv)) {
 #ifdef PR_LOGGING
-        if (PR_LOG_TEST(gLog, PR_LOG_ERROR)) {
+        if (PR_LOG_TEST(gContentSinkLog, PR_LOG_ERROR)) {
             nsAutoString anodeC;
             aNodeInfo->GetName(anodeC);
-            PR_LOG(gLog, PR_LOG_ERROR,
+            PR_LOG(gContentSinkLog, PR_LOG_ERROR,
                    ("xul: unable to create element '%s' at line %d",
                     NS_ConvertUTF16toUTF8(anodeC).get(),
                     -1)); // XXX pass in line number
@@ -800,7 +790,7 @@ XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes,
 }
 
 nsresult
-XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes, 
+XULContentSinkImpl::OpenTag(const char16_t** aAttributes, 
                             const uint32_t aAttrLen,
                             const uint32_t aLineNumber,
                             nsINodeInfo *aNodeInfo)
@@ -813,10 +803,10 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
 
     if (NS_FAILED(rv)) {
 #ifdef PR_LOGGING
-        if (PR_LOG_TEST(gLog, PR_LOG_ERROR)) {
+        if (PR_LOG_TEST(gContentSinkLog, PR_LOG_ERROR)) {
             nsAutoString anodeC;
             aNodeInfo->GetName(anodeC);
-            PR_LOG(gLog, PR_LOG_ERROR,
+            PR_LOG(gContentSinkLog, PR_LOG_ERROR,
                    ("xul: unable to create element '%s' at line %d",
                     NS_ConvertUTF16toUTF8(anodeC).get(),
                     aLineNumber));
@@ -865,7 +855,7 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
 }
 
 nsresult
-XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
+XULContentSinkImpl::OpenScript(const char16_t** aAttributes,
                                const uint32_t aLineNumber)
 {
   uint32_t langID = nsIProgrammingLanguage::JAVASCRIPT;
@@ -944,9 +934,9 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
 
   // Don't process scripts that aren't known
   if (langID != nsIProgrammingLanguage::UNKNOWN) {
-      nsIScriptGlobalObject* globalObject = nullptr; // borrowed reference
+      nsCOMPtr<nsIScriptGlobalObject> globalObject;
       if (doc)
-          globalObject = doc->GetScriptGlobalObject();
+          globalObject = do_QueryInterface(doc->GetWindow());
       nsRefPtr<nsXULPrototypeScript> script =
           new nsXULPrototypeScript(aLineNumber, version);
       if (! script)
@@ -983,8 +973,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
           // file right away.  Otherwise we'll end up reloading the script and
           // corrupting the FastLoad file trying to serialize it, in the case
           // where it's already there.
-          if (globalObject)
-                script->DeserializeOutOfLine(nullptr, globalObject);
+          script->DeserializeOutOfLine(nullptr, mPrototype);
       }
 
       nsPrototypeArray* children = nullptr;
@@ -1005,7 +994,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
 }
 
 nsresult
-XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes, 
+XULContentSinkImpl::AddAttributes(const char16_t** aAttributes, 
                                   const uint32_t aAttrLen, 
                                   nsXULPrototypeElement* aElement)
 {
@@ -1034,7 +1023,7 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
       NS_ENSURE_SUCCESS(rv, rv);
 
 #ifdef PR_LOGGING
-      if (PR_LOG_TEST(gLog, PR_LOG_DEBUG)) {
+      if (PR_LOG_TEST(gContentSinkLog, PR_LOG_DEBUG)) {
           nsAutoString extraWhiteSpace;
           int32_t cnt = mContextStack.Depth();
           while (--cnt >= 0)
@@ -1042,7 +1031,7 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
           nsAutoString qnameC,valueC;
           qnameC.Assign(aAttributes[0]);
           valueC.Assign(aAttributes[1]);
-          PR_LOG(gLog, PR_LOG_DEBUG,
+          PR_LOG(gContentSinkLog, PR_LOG_DEBUG,
                  ("xul: %.5d. %s    %s=%s",
                   -1, // XXX pass in line number
                   NS_ConvertUTF16toUTF8(extraWhiteSpace).get(),
@@ -1056,12 +1045,12 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
 }
 
 nsresult
-XULContentSinkImpl::AddText(const PRUnichar* aText, 
+XULContentSinkImpl::AddText(const char16_t* aText, 
                             int32_t aLength)
 {
   // Create buffer when we first need it
   if (0 == mTextSize) {
-      mText = (PRUnichar *) moz_malloc(sizeof(PRUnichar) * 4096);
+      mText = (char16_t *) moz_malloc(sizeof(char16_t) * 4096);
       if (nullptr == mText) {
           return NS_ERROR_OUT_OF_MEMORY;
       }
@@ -1084,13 +1073,13 @@ XULContentSinkImpl::AddText(const PRUnichar* aText,
       }
       else {
         mTextSize += aLength;
-        mText = (PRUnichar *) moz_realloc(mText, sizeof(PRUnichar) * mTextSize);
+        mText = (char16_t *) moz_realloc(mText, sizeof(char16_t) * mTextSize);
         if (nullptr == mText) {
             return NS_ERROR_OUT_OF_MEMORY;
         }
       }
     }
-    memcpy(&mText[mTextLength],aText + offset, sizeof(PRUnichar) * amount);
+    memcpy(&mText[mTextLength],aText + offset, sizeof(char16_t) * amount);
     
     mTextLength += amount;
     offset += amount;
