@@ -41,19 +41,18 @@
 
 struct nsCSSValueSharedList;
 
-using namespace mozilla::dom;
-using namespace mozilla::gfx;
-
 namespace mozilla {
 namespace layers {
 
+using namespace mozilla::gfx;
+
 enum Op { Resolve, Detach };
 
 static bool
-IsSameDimension(ScreenOrientation o1, ScreenOrientation o2)
+IsSameDimension(dom::ScreenOrientation o1, dom::ScreenOrientation o2)
 {
-  bool isO1portrait = (o1 == eScreenOrientation_PortraitPrimary || o1 == eScreenOrientation_PortraitSecondary);
-  bool isO2portrait = (o2 == eScreenOrientation_PortraitPrimary || o2 == eScreenOrientation_PortraitSecondary);
+  bool isO1portrait = (o1 == dom::eScreenOrientation_PortraitPrimary || o1 == dom::eScreenOrientation_PortraitSecondary);
+  bool isO2portrait = (o2 == dom::eScreenOrientation_PortraitPrimary || o2 == dom::eScreenOrientation_PortraitSecondary);
   return !(isO1portrait ^ isO2portrait);
 }
 
@@ -73,8 +72,8 @@ WalkTheTree(Layer* aLayer,
     if (const CompositorParent::LayerTreeState* state = CompositorParent::GetIndirectShadowTree(ref->GetReferentId())) {
       if (Layer* referent = state->mRoot) {
         if (!ref->GetVisibleRegion().IsEmpty()) {
-          ScreenOrientation chromeOrientation = aTargetConfig.orientation();
-          ScreenOrientation contentOrientation = state->mTargetConfig.orientation();
+          dom::ScreenOrientation chromeOrientation = aTargetConfig.orientation();
+          dom::ScreenOrientation contentOrientation = state->mTargetConfig.orientation();
           if (!IsSameDimension(chromeOrientation, contentOrientation) &&
               ContentMightReflowOnOrientationChange(aTargetConfig.clientBounds())) {
             aReady = false;
@@ -98,6 +97,11 @@ WalkTheTree(Layer* aLayer,
 void
 AsyncCompositionManager::ResolveRefLayers()
 {
+  if (!mLayerManager->GetRoot()) {
+    return;
+  }
+
+  mReadyForCompose = true;
   WalkTheTree<Resolve>(mLayerManager->GetRoot(),
                        mReadyForCompose,
                        mTargetConfig);
@@ -106,6 +110,9 @@ AsyncCompositionManager::ResolveRefLayers()
 void
 AsyncCompositionManager::DetachRefLayers()
 {
+  if (!mLayerManager->GetRoot()) {
+    return;
+  }
   WalkTheTree<Detach>(mLayerManager->GetRoot(),
                       mReadyForCompose,
                       mTargetConfig);
@@ -231,6 +238,7 @@ void
 AsyncCompositionManager::AlignFixedAndStickyLayers(Layer* aLayer,
                                                    Layer* aTransformedSubtreeRoot,
                                                    const Matrix4x4& aPreviousTransformForRoot,
+                                                   const Matrix4x4& aCurrentTransformForRoot,
                                                    const LayerMargin& aFixedLayerMargins)
 {
   bool isRootFixed = aLayer->GetIsFixedPosition() &&
@@ -238,7 +246,7 @@ AsyncCompositionManager::AlignFixedAndStickyLayers(Layer* aLayer,
   bool isStickyForSubtree = aLayer->GetIsStickyPosition() &&
     aTransformedSubtreeRoot->AsContainerLayer() &&
     aLayer->GetStickyScrollContainerId() ==
-      aTransformedSubtreeRoot->AsContainerLayer()->GetFrameMetrics().mScrollId;
+      aTransformedSubtreeRoot->AsContainerLayer()->GetFrameMetrics().GetScrollId();
   if (aLayer != aTransformedSubtreeRoot && (isRootFixed || isStickyForSubtree)) {
     // Insert a translation so that the position of the anchor point is the same
     // before and after the change to the transform of aTransformedSubtreeRoot.
@@ -254,7 +262,7 @@ AsyncCompositionManager::AlignFixedAndStickyLayers(Layer* aLayer,
     Matrix oldRootTransform;
     Matrix newRootTransform;
     if (!aPreviousTransformForRoot.Is2D(&oldRootTransform) ||
-        !aTransformedSubtreeRoot->GetLocalTransform().Is2D(&newRootTransform)) {
+        !aCurrentTransformForRoot.Is2D(&newRootTransform)) {
       return;
     }
 
@@ -327,19 +335,20 @@ AsyncCompositionManager::AlignFixedAndStickyLayers(Layer* aLayer,
   }
 
   // Fixed layers are relative to their nearest scrollable layer, so when we
-  // encounter a scrollable layer, reset the transform to that layer and remove
-  // the fixed margins.
+  // encounter a scrollable layer, bail. ApplyAsyncContentTransformToTree will
+  // have already recursed on this layer and called AlignFixedAndStickyLayers
+  // on it with its own transforms.
   if (aLayer->AsContainerLayer() &&
       aLayer->AsContainerLayer()->GetFrameMetrics().IsScrollable() &&
       aLayer != aTransformedSubtreeRoot) {
-    AlignFixedAndStickyLayers(aLayer, aLayer, aLayer->GetTransform(), LayerMargin(0, 0, 0, 0));
     return;
   }
 
   for (Layer* child = aLayer->GetFirstChild();
        child; child = child->GetNextSibling()) {
     AlignFixedAndStickyLayers(child, aTransformedSubtreeRoot,
-                              aPreviousTransformForRoot, aFixedLayerMargins);
+                              aPreviousTransformForRoot,
+                              aCurrentTransformForRoot, aFixedLayerMargins);
   }
 }
 
@@ -407,31 +416,52 @@ SampleAnimations(Layer* aLayer, TimeStamp aPoint)
     Animation& animation = animations[i];
     AnimData& animData = animationData[i];
 
-    double numIterations = animation.numIterations() != -1 ?
-      animation.numIterations() : NS_IEEEPositiveInfinity();
-    double positionInIteration =
-      ElementAnimations::GetPositionInIteration(aPoint - animation.startTime(),
-                                                animation.duration(),
-                                                numIterations,
-                                                animation.direction());
+    activeAnimations = true;
 
-    NS_ABORT_IF_FALSE(0.0 <= positionInIteration &&
-                      positionInIteration <= 1.0,
-                      "position should be in [0-1]");
+    TimeDuration elapsedDuration = aPoint - animation.startTime();
+    // Skip animations that are yet to start.
+    //
+    // Currently, this should only happen when the refresh driver is under test
+    // control and is made to produce a time in the past or is restored from
+    // test control causing it to jump backwards in time.
+    //
+    // Since activeAnimations is true, this could mean we keep compositing
+    // unnecessarily during the delay, but so long as this only happens while
+    // the refresh driver is under test control that should be ok.
+    if (elapsedDuration.ToSeconds() < 0) {
+      continue;
+    }
+
+    AnimationTiming timing;
+    timing.mIterationDuration = animation.duration();
+    timing.mIterationCount = animation.iterationCount();
+    timing.mDirection = animation.direction();
+    // Animations typically only run on the compositor during their active
+    // interval but if we end up sampling them outside that range (for
+    // example, while they are waiting to be removed) we currently just
+    // assume that we should fill.
+    timing.mFillMode = NS_STYLE_ANIMATION_FILL_MODE_BOTH;
+
+    ComputedTiming computedTiming =
+      ElementAnimation::GetComputedTimingAt(elapsedDuration, timing);
+
+    NS_ABORT_IF_FALSE(0.0 <= computedTiming.mTimeFraction &&
+                      computedTiming.mTimeFraction <= 1.0,
+                      "time fraction should be in [0-1]");
 
     int segmentIndex = 0;
     AnimationSegment* segment = animation.segments().Elements();
-    while (segment->endPortion() < positionInIteration) {
+    while (segment->endPortion() < computedTiming.mTimeFraction) {
       ++segment;
       ++segmentIndex;
     }
 
-    double positionInSegment = (positionInIteration - segment->startPortion()) /
-                                 (segment->endPortion() - segment->startPortion());
+    double positionInSegment =
+      (computedTiming.mTimeFraction - segment->startPortion()) /
+      (segment->endPortion() - segment->startPortion());
 
-    double portion = animData.mFunctions[segmentIndex]->GetValue(positionInSegment);
-
-    activeAnimations = true;
+    double portion =
+      animData.mFunctions[segmentIndex]->GetValue(positionInSegment);
 
     // interpolate the property
     Animatable interpolatedValue;
@@ -469,6 +499,15 @@ SampleAnimations(Layer* aLayer, TimeStamp aPoint)
   return activeAnimations;
 }
 
+Matrix4x4
+CombineWithCSSTransform(const gfx3DMatrix& treeTransform, Layer* aLayer)
+{
+  Matrix4x4 result;
+  ToMatrix4x4(treeTransform, result);
+  result = result * aLayer->GetTransform();
+  return result;
+}
+
 bool
 AsyncCompositionManager::ApplyAsyncContentTransformToTree(TimeStamp aCurrentFrame,
                                                           Layer *aLayer,
@@ -490,12 +529,13 @@ AsyncCompositionManager::ApplyAsyncContentTransformToTree(TimeStamp aCurrentFram
     LayerComposite* layerComposite = aLayer->AsLayerComposite();
     Matrix4x4 oldTransform = aLayer->GetTransform();
 
-    ViewTransform treeTransform;
+    ViewTransform treeTransformWithoutOverscroll, overscrollTransform;
     ScreenPoint scrollOffset;
     *aWantNextFrame |=
       controller->SampleContentTransformForFrame(aCurrentFrame,
-                                                 &treeTransform,
-                                                 scrollOffset);
+                                                 &treeTransformWithoutOverscroll,
+                                                 scrollOffset,
+                                                 &overscrollTransform);
 
     const FrameMetrics& metrics = container->GetFrameMetrics();
     CSSToLayerScale paintScale = metrics.LayersPixelsPerCSSPixel();
@@ -503,9 +543,9 @@ AsyncCompositionManager::ApplyAsyncContentTransformToTree(TimeStamp aCurrentFram
                         metrics.mDisplayPort : metrics.mCriticalDisplayPort);
     LayerMargin fixedLayerMargins(0, 0, 0, 0);
     ScreenPoint offset(0, 0);
-    SyncFrameMetrics(scrollOffset, treeTransform.mScale.scale, metrics.mScrollableRect,
-                     mLayersUpdated, displayPort, paintScale,
-                     mIsFirstPaint, fixedLayerMargins, offset);
+    SyncFrameMetrics(scrollOffset, treeTransformWithoutOverscroll.mScale.scale,
+                     metrics.mScrollableRect, mLayersUpdated, displayPort,
+                     paintScale, mIsFirstPaint, fixedLayerMargins, offset);
 
     mIsFirstPaint = false;
     mLayersUpdated = false;
@@ -513,9 +553,8 @@ AsyncCompositionManager::ApplyAsyncContentTransformToTree(TimeStamp aCurrentFram
     // Apply the render offset
     mLayerManager->GetCompositor()->SetScreenRenderOffset(offset);
 
-    Matrix4x4 transform;
-    ToMatrix4x4(gfx3DMatrix(treeTransform), transform);
-    transform = transform * aLayer->GetTransform();
+    Matrix4x4 transform = CombineWithCSSTransform(
+        treeTransformWithoutOverscroll * overscrollTransform, aLayer);
 
     // GetTransform already takes the pre- and post-scale into account.  Since we
     // will apply the pre- and post-scale again when computing the effective
@@ -535,13 +574,20 @@ AsyncCompositionManager::ApplyAsyncContentTransformToTree(TimeStamp aCurrentFram
     LayoutDeviceToLayerScale resolution = metrics.mCumulativeResolution;
     oldTransform.Scale(resolution.scale, resolution.scale, 1);
 
-    AlignFixedAndStickyLayers(aLayer, aLayer, oldTransform, fixedLayerMargins);
+    // For the purpose of aligning fixed and sticky layers, we disregard
+    // the overscroll transform when computing the 'aCurrentTransformForRoot'
+    // parameter. This ensures that the overscroll transform is not unapplied,
+    // and therefore that the visual effect applies to fixed and sticky layers.
+    Matrix4x4 transformWithoutOverscroll = CombineWithCSSTransform(
+        treeTransformWithoutOverscroll, aLayer);
+    AlignFixedAndStickyLayers(aLayer, aLayer, oldTransform,
+                              transformWithoutOverscroll, fixedLayerMargins);
 
     appliedTransform = true;
   }
 
   if (container->GetScrollbarDirection() != Layer::NONE) {
-    ApplyAsyncTransformToScrollbar(container);
+    ApplyAsyncTransformToScrollbar(aCurrentFrame, container);
   }
   return appliedTransform;
 }
@@ -560,69 +606,149 @@ LayerHasNonContainerDescendants(ContainerLayer* aContainer)
   return false;
 }
 
-void
-AsyncCompositionManager::ApplyAsyncTransformToScrollbar(ContainerLayer* aLayer)
+static bool
+LayerIsContainerForScrollbarTarget(Layer* aTarget, ContainerLayer* aScrollbar)
 {
-  // If this layer corresponds to a scrollbar, then search backwards through the
-  // siblings until we find the container layer with the right ViewID; this is
-  // the content that this scrollbar is for. Pick up the transient async transform
-  // from that layer and use it to update the scrollbar position.
-  // Note that it is possible that the content layer is no longer there; in
-  // this case we don't need to do anything because there can't be an async
-  // transform on the content.
+  if (!aTarget->AsContainerLayer()) {
+    return false;
+  }
+  AsyncPanZoomController* apzc = aTarget->AsContainerLayer()->GetAsyncPanZoomController();
+  if (!apzc) {
+    return false;
+  }
+  const FrameMetrics& metrics = aTarget->AsContainerLayer()->GetFrameMetrics();
+  if (metrics.GetScrollId() != aScrollbar->GetScrollbarTargetContainerId()) {
+    return false;
+  }
+  return true;
+}
+
+static void
+ApplyAsyncTransformToScrollbarForContent(TimeStamp aCurrentFrame, ContainerLayer* aScrollbar,
+                                         Layer* aContent, bool aScrollbarIsChild)
+{
+  ContainerLayer* content = aContent->AsContainerLayer();
+
   // We only apply the transform if the scroll-target layer has non-container
   // children (i.e. when it has some possibly-visible content). This is to
   // avoid moving scroll-bars in the situation that only a scroll information
   // layer has been built for a scroll frame, as this would result in a
   // disparity between scrollbars and visible content.
-  for (Layer* scrollTarget = aLayer->GetPrevSibling();
-       scrollTarget;
-       scrollTarget = scrollTarget->GetPrevSibling()) {
-    if (!scrollTarget->AsContainerLayer()) {
-      continue;
-    }
-    AsyncPanZoomController* apzc = scrollTarget->AsContainerLayer()->GetAsyncPanZoomController();
-    if (!apzc) {
-      continue;
-    }
-    const FrameMetrics& metrics = scrollTarget->AsContainerLayer()->GetFrameMetrics();
-    if (metrics.mScrollId != aLayer->GetScrollbarTargetContainerId()) {
-      continue;
-    }
-    if (!LayerHasNonContainerDescendants(scrollTarget->AsContainerLayer())) {
-      return;
-    }
-
-    gfx3DMatrix asyncTransform = gfx3DMatrix(apzc->GetCurrentAsyncTransform());
-    gfx3DMatrix nontransientTransform = apzc->GetNontransientAsyncTransform();
-    gfx3DMatrix transientTransform = asyncTransform * nontransientTransform.Inverse();
-
-    Matrix4x4 scrollbarTransform;
-    if (aLayer->GetScrollbarDirection() == Layer::VERTICAL) {
-      float scale = metrics.CalculateCompositedRectInCssPixels().height / metrics.mScrollableRect.height;
-      scrollbarTransform = scrollbarTransform * Matrix4x4().Scale(1.f, 1.f / transientTransform.GetYScale(), 1.f);
-      scrollbarTransform = scrollbarTransform * Matrix4x4().Translate(0, -transientTransform._42 * scale, 0);
-    }
-    if (aLayer->GetScrollbarDirection() == Layer::HORIZONTAL) {
-      float scale = metrics.CalculateCompositedRectInCssPixels().width / metrics.mScrollableRect.width;
-      scrollbarTransform = scrollbarTransform * Matrix4x4().Scale(1.f / transientTransform.GetXScale(), 1.f, 1.f);
-      scrollbarTransform = scrollbarTransform * Matrix4x4().Translate(-transientTransform._41 * scale, 0, 0);
-    }
-
-    Matrix4x4 transform = scrollbarTransform * aLayer->GetTransform();
-    // GetTransform already takes the pre- and post-scale into account.  Since we
-    // will apply the pre- and post-scale again when computing the effective
-    // transform, we must apply the inverses here.
-    transform.Scale(1.0f/aLayer->GetPreXScale(),
-                    1.0f/aLayer->GetPreYScale(),
-                    1);
-    transform = transform * Matrix4x4().Scale(1.0f/aLayer->GetPostXScale(),
-                                              1.0f/aLayer->GetPostYScale(),
-                                              1);
-    aLayer->AsLayerComposite()->SetShadowTransform(transform);
-
+  if (!LayerHasNonContainerDescendants(content)) {
     return;
   }
+
+  const FrameMetrics& metrics = content->GetFrameMetrics();
+  AsyncPanZoomController* apzc = content->GetAsyncPanZoomController();
+
+  if (aScrollbarIsChild) {
+    // Because we try to apply the scrollbar transform before we apply the async transform on
+    // the actual content, we need to ensure that the APZC has updated any pending animations
+    // to the current frame timestamp before we extract the transforms from it. The code in this
+    // block accomplishes that and throws away the temp variables.
+    // TODO: it might be cleaner to do a pass through the layer tree to advance all the APZC
+    // transforms before updating the layer shadow transforms. That will allow removal of this code.
+    ViewTransform treeTransform;
+    ScreenPoint scrollOffset;
+    apzc->SampleContentTransformForFrame(aCurrentFrame, &treeTransform, scrollOffset);
+  }
+
+  gfx3DMatrix asyncTransform = gfx3DMatrix(apzc->GetCurrentAsyncTransform());
+  gfx3DMatrix nontransientTransform = apzc->GetNontransientAsyncTransform();
+  gfx3DMatrix transientTransform = asyncTransform * nontransientTransform.Inverse();
+
+  // |transientTransform| represents the amount by which we have scrolled and
+  // zoomed since the last paint. Because the scrollbar was sized and positioned based
+  // on the painted content, we need to adjust it based on transientTransform so that
+  // it reflects what the user is actually seeing now.
+  // - The scroll thumb needs to be scaled in the direction of scrolling by the inverse
+  //   of the transientTransform scale (representing the zoom). This is because zooming
+  //   in decreases the fraction of the whole scrollable rect that is in view.
+  // - It needs to be translated in opposite direction of the transientTransform
+  //   translation (representing the scroll). This is because scrolling down, which
+  //   translates the layer content up, should result in moving the scroll thumb down.
+  //   The amount of the translation to the scroll thumb should be such that the ratio
+  //   of the translation to the size of the scroll port is the same as the ratio
+  //   of the scroll amount to the size of the scrollable rect.
+  Matrix4x4 scrollbarTransform;
+  if (aScrollbar->GetScrollbarDirection() == Layer::VERTICAL) {
+    float scale = metrics.CalculateCompositedSizeInCssPixels().height / metrics.mScrollableRect.height;
+    scrollbarTransform = scrollbarTransform * Matrix4x4().Scale(1.f, 1.f / transientTransform.GetYScale(), 1.f);
+    scrollbarTransform = scrollbarTransform * Matrix4x4().Translate(0, -transientTransform._42 * scale, 0);
+  }
+  if (aScrollbar->GetScrollbarDirection() == Layer::HORIZONTAL) {
+    float scale = metrics.CalculateCompositedSizeInCssPixels().width / metrics.mScrollableRect.width;
+    scrollbarTransform = scrollbarTransform * Matrix4x4().Scale(1.f / transientTransform.GetXScale(), 1.f, 1.f);
+    scrollbarTransform = scrollbarTransform * Matrix4x4().Translate(-transientTransform._41 * scale, 0, 0);
+  }
+
+  Matrix4x4 transform = scrollbarTransform * aScrollbar->GetTransform();
+
+  if (aScrollbarIsChild) {
+    // If the scrollbar layer is a child of the content it is a scrollbar for, then we
+    // need to do an extra untransform to cancel out the transient async transform on
+    // the content. This is needed because otherwise that transient async transform is
+    // part of the effective transform of this scrollbar, and the scrollbar will jitter
+    // as the content scrolls.
+    Matrix4x4 targetUntransform;
+    ToMatrix4x4(transientTransform.Inverse(), targetUntransform);
+    transform = transform * targetUntransform;
+  }
+
+  // GetTransform already takes the pre- and post-scale into account.  Since we
+  // will apply the pre- and post-scale again when computing the effective
+  // transform, we must apply the inverses here.
+  transform.Scale(1.0f/aScrollbar->GetPreXScale(),
+                  1.0f/aScrollbar->GetPreYScale(),
+                  1);
+  transform = transform * Matrix4x4().Scale(1.0f/aScrollbar->GetPostXScale(),
+                                            1.0f/aScrollbar->GetPostYScale(),
+                                            1);
+  aScrollbar->AsLayerComposite()->SetShadowTransform(transform);
+}
+
+static Layer*
+FindScrolledLayerForScrollbar(ContainerLayer* aLayer, bool* aOutIsAncestor)
+{
+  // Search all siblings of aLayer and of its ancestors.
+  for (Layer* ancestor = aLayer; ancestor; ancestor = ancestor->GetParent()) {
+    for (Layer* scrollTarget = ancestor;
+         scrollTarget;
+         scrollTarget = scrollTarget->GetPrevSibling()) {
+      if (scrollTarget != aLayer &&
+          LayerIsContainerForScrollbarTarget(scrollTarget, aLayer)) {
+        *aOutIsAncestor = (scrollTarget == ancestor);
+        return scrollTarget;
+      }
+    }
+    for (Layer* scrollTarget = ancestor->GetNextSibling();
+         scrollTarget;
+         scrollTarget = scrollTarget->GetNextSibling()) {
+      if (LayerIsContainerForScrollbarTarget(scrollTarget, aLayer)) {
+        *aOutIsAncestor = false;
+        return scrollTarget;
+      }
+    }
+  }
+  return nullptr;
+}
+
+void
+AsyncCompositionManager::ApplyAsyncTransformToScrollbar(TimeStamp aCurrentFrame, ContainerLayer* aLayer)
+{
+  // If this layer corresponds to a scrollbar, then there should be a layer that
+  // is a previous sibling or a parent that has a matching ViewID on its FrameMetrics.
+  // That is the content that this scrollbar is for. We pick up the transient
+  // async transform from that layer and use it to update the scrollbar position.
+  // Note that it is possible that the content layer is no longer there; in
+  // this case we don't need to do anything because there can't be an async
+  // transform on the content.
+  bool isAncestor = false;
+  Layer* scrollTarget = FindScrolledLayerForScrollbar(aLayer, &isAncestor);
+  if (scrollTarget) {
+    ApplyAsyncTransformToScrollbarForContent(aCurrentFrame, aLayer, scrollTarget,
+                                             isAncestor);
+  }
 }
 
 void
@@ -642,7 +768,7 @@ AsyncCompositionManager::TransformScrollableLayer(Layer* aLayer)
 
   CSSToLayerScale geckoZoom = metrics.LayersPixelsPerCSSPixel();
 
-  LayerIntPoint scrollOffsetLayerPixels = RoundedToInt(metrics.mScrollOffset * geckoZoom);
+  LayerIntPoint scrollOffsetLayerPixels = RoundedToInt(metrics.GetScrollOffset() * geckoZoom);
 
   if (mIsFirstPaint) {
     mContentRect = metrics.mScrollableRect;
@@ -674,7 +800,7 @@ AsyncCompositionManager::TransformScrollableLayer(Layer* aLayer)
   // however, we can assume there is no async zooming in progress and so the following statement
   // works fine.
   CSSToScreenScale userZoom(metrics.mDevPixelsPerCSSPixel * metrics.mCumulativeResolution * LayerToScreenScale(1));
-  ScreenPoint userScroll = metrics.mScrollOffset * userZoom;
+  ScreenPoint userScroll = metrics.GetScrollOffset() * userZoom;
   SyncViewportInfo(displayPort, geckoZoom, mLayersUpdated,
                    userScroll, userZoom, fixedLayerMargins,
                    offset);
@@ -693,7 +819,7 @@ AsyncCompositionManager::TransformScrollableLayer(Layer* aLayer)
 
   LayerPoint geckoScroll(0, 0);
   if (metrics.IsScrollable()) {
-    geckoScroll = metrics.mScrollOffset * geckoZoom;
+    geckoScroll = metrics.GetScrollOffset() * geckoZoom;
   }
 
   LayerPoint translation = (userScroll / zoomAdjust) - geckoScroll;
@@ -758,13 +884,16 @@ AsyncCompositionManager::TransformScrollableLayer(Layer* aLayer)
 
   // Make sure fixed position layers don't move away from their anchor points
   // when we're asynchronously panning or zooming
-  AlignFixedAndStickyLayers(aLayer, aLayer, oldTransform, fixedLayerMargins);
+  AlignFixedAndStickyLayers(aLayer, aLayer, oldTransform,
+                            aLayer->GetLocalTransform(), fixedLayerMargins);
 }
 
 bool
 AsyncCompositionManager::TransformShadowTree(TimeStamp aCurrentFrame)
 {
-  PROFILER_LABEL("AsyncCompositionManager", "TransformShadowTree");
+  PROFILER_LABEL("AsyncCompositionManager", "TransformShadowTree",
+    js::ProfileEntry::Category::GRAPHICS);
+
   Layer* root = mLayerManager->GetRoot();
   if (!root) {
     return false;
