@@ -14,6 +14,7 @@
 #include "X11/Xlib.h"
 #endif
 
+class gfxImageSurface;
 class gfxFontconfigUtils;
 class QWindow;
 
@@ -42,13 +43,9 @@ public:
 
     virtual nsresult UpdateFontList() MOZ_OVERRIDE;
 
-    virtual nsresult ResolveFontName(const nsAString& aFontName,
-                                     FontResolverCallback aCallback,
-                                     void *aClosure, bool& aAborted) MOZ_OVERRIDE;
-
     virtual nsresult GetStandardFamilyName(const nsAString& aFontName, nsAString& aFamilyName) MOZ_OVERRIDE;
 
-    virtual gfxFontGroup *CreateFontGroup(const nsAString &aFamilies,
+    virtual gfxFontGroup *CreateFontGroup(const mozilla::FontFamilyList& aFontFamilyList,
                                           const gfxFontStyle *aStyle,
                                           gfxUserFontSet* aUserFontSet) MOZ_OVERRIDE;
 
@@ -92,28 +89,12 @@ protected:
     static gfxFontconfigUtils *sFontconfigUtils;
 
 private:
-
-    bool UseXRender() {
-#if defined(MOZ_X11)
-        if (GetContentBackend() != mozilla::gfx::BackendType::NONE &&
-            GetContentBackend() != mozilla::gfx::BackendType::CAIRO)
-            return false;
-
-        return sUseXRender;
-#else
-        return false;
-#endif
-    }
-
     virtual void GetPlatformCMSOutputProfile(void *&mem, size_t &size) MOZ_OVERRIDE;
 
     // TODO: unify this with mPrefFonts (NB: holds families, not fonts) in gfxPlatformFontList
     nsDataHashtable<nsCStringHashKey, nsTArray<nsRefPtr<gfxFontEntry> > > mPrefFonts;
 
     int mScreenDepth;
-#ifdef MOZ_X11
-    static bool sUseXRender;
-#endif
 };
 
 #endif /* GFX_PLATFORM_QT_H */
