@@ -7,8 +7,10 @@
 #ifndef __nsScriptableUConv_h_
 #define __nsScriptableUConv_h_
 
-#include "nsICharsetConverterManager.h"
 #include "nsIScriptableUConv.h"
+#include "nsCOMPtr.h"
+#include "nsIUnicodeDecoder.h"
+#include "nsIUnicodeEncoder.h"
 
 class nsScriptableUnicodeConverter : public nsIScriptableUnicodeConverter
 {
@@ -25,9 +27,9 @@ protected:
   nsCOMPtr<nsIUnicodeDecoder> mDecoder;
   bool mIsInternal;
 
-  nsresult FinishWithLength(char **_retval, PRInt32* aLength);
+  nsresult FinishWithLength(char **_retval, int32_t* aLength);
   nsresult ConvertFromUnicodeWithLength(const nsAString& aSrc,
-                                        PRInt32* aOutLen,
+                                        int32_t* aOutLen,
                                         char **_retval);
 
 
