@@ -5,9 +5,8 @@
 #ifndef nsISO2022KRToUnicode_h__
 #define nsISO2022KRToUnicode_h__
 #include "nsUCSupport.h"
+#include "mozilla/Telemetry.h"
 
-
- 
 class nsISO2022KRToUnicode : public nsBasicDecoderSupport
 {
 public:
@@ -18,15 +17,16 @@ public:
     mData = 0;
     mEUCKRDecoder = nullptr;
     mRunLength = 0;
+    mozilla::Telemetry::Accumulate(
+      mozilla::Telemetry::DECODER_INSTANTIATED_ISO2022KR, true);
   }
 
   virtual ~nsISO2022KRToUnicode()
   {
-    NS_IF_RELEASE(mEUCKRDecoder);
   }
 
   NS_IMETHOD Convert(const char * aSrc, int32_t * aSrcLength,
-     PRUnichar * aDest, int32_t * aDestLength) ;
+     char16_t * aDest, int32_t * aDestLength) ;
   
   NS_IMETHOD GetMaxLength(const char * aSrc, int32_t aSrcLength,
      int32_t * aDestLength) 
@@ -60,6 +60,6 @@ private:
   // Length of non-ASCII run
   uint32_t mRunLength;
 
-  nsIUnicodeDecoder *mEUCKRDecoder;
+  nsCOMPtr<nsIUnicodeDecoder> mEUCKRDecoder;
 };
 #endif // nsISO2022KRToUnicode_h__
