@@ -15,10 +15,12 @@
 
 #include "ds/BitArray.h"
 #include "gc/Heap.h"
+#include "gc/Memory.h"
 #include "js/GCAPI.h"
 #include "js/HashTable.h"
 #include "js/HeapAPI.h"
 #include "js/Value.h"
+#include "js/Vector.h"
 
 namespace JS {
 struct Zone;
@@ -28,12 +30,18 @@ namespace js {
 
 class ObjectElements;
 class HeapSlot;
+void SetGCZeal(JSRuntime *, uint8_t, uint32_t);
 
 namespace gc {
 class Cell;
+class Collector;
 class MinorCollectionTracer;
 } /* namespace gc */
 
+namespace types {
+struct TypeObject;
+}
+
 namespace jit {
 class CodeGenerator;
 class MacroAssembler;
@@ -44,36 +52,49 @@ class BaselineCompiler;
 class Nursery
 {
   public:
-    static const int NumNurseryChunks = 16;
-    static const int LastNurseryChunk = NumNurseryChunks - 1;
     static const size_t Alignment = gc::ChunkSize;
-    static const size_t NurserySize = gc::ChunkSize * NumNurseryChunks;
+    static const size_t ChunkShift = gc::ChunkShift;
 
     explicit Nursery(JSRuntime *rt)
       : runtime_(rt),
         position_(0),
+        currentStart_(0),
         currentEnd_(0),
+        heapStart_(0),
+        heapEnd_(0),
         currentChunk_(0),
-        numActiveChunks_(0)
+        numActiveChunks_(0),
+        numNurseryChunks_(0)
     {}
     ~Nursery();
 
-    bool init();
+    bool init(uint32_t numNurseryChunks);
+
+    bool exists() const { return numNurseryChunks_ != 0; }
+    size_t numChunks() const { return numNurseryChunks_; }
+    size_t nurserySize() const { return numNurseryChunks_ << ChunkShift; }
 
     void enable();
     void disable();
     bool isEnabled() const { return numActiveChunks_ != 0; }
 
-    template <typename T>
-    JS_ALWAYS_INLINE bool isInside(const T *p) const {
-        return gc::IsInsideNursery((JS::shadow::Runtime *)runtime_, p);
+    /* Return true if no allocations have been made since the last collection. */
+    bool isEmpty() const;
+
+    /*
+     * Check whether an arbitrary pointer is within the nursery. This is
+     * slower than IsInsideNursery(Cell*), but works on all types of pointers.
+     */
+    MOZ_ALWAYS_INLINE bool isInside(gc::Cell *cellp) const MOZ_DELETE;
+    MOZ_ALWAYS_INLINE bool isInside(const void *p) const {
+        return uintptr_t(p) >= heapStart_ && uintptr_t(p) < heapEnd_;
     }
 
     /*
-     * Allocate and return a pointer to a new GC thing. Returns nullptr if the
-     * Nursery is full.
+     * Allocate and return a pointer to a new GC object with its |slots|
+     * pointer pre-filled. Returns nullptr if the Nursery is full.
      */
-    void *allocate(size_t size);
+    JSObject *allocateObject(JSContext *cx, size_t size, size_t numDynamic);
 
     /* Allocate a slots array for the given object. */
     HeapSlot *allocateSlots(JSContext *cx, JSObject *obj, uint32_t nslots);
@@ -92,11 +113,13 @@ class Nursery
     /* Free a slots array. */
     void freeSlots(JSContext *cx, HeapSlot *slots);
 
-    /* Add a slots to our tracking list if it is out-of-line. */
-    void notifyInitialSlots(gc::Cell *cell, HeapSlot *slots);
+    typedef Vector<types::TypeObject *, 0, SystemAllocPolicy> TypeObjectList;
 
-    /* Do a minor collection. */
-    void collect(JSRuntime *rt, JS::gcreason::Reason reason);
+    /*
+     * Do a minor collection, optionally specifying a list to store types which
+     * should be pretenured afterwards.
+     */
+    void collect(JSRuntime *rt, JS::gcreason::Reason reason, TypeObjectList *pretenureTypes);
 
     /*
      * Check if the thing at |*ref| in the Nursery has been forwarded. If so,
@@ -104,20 +127,36 @@ class Nursery
      * returns false and leaves |*ref| unset.
      */
     template <typename T>
-    JS_ALWAYS_INLINE bool getForwardedPointer(T **ref);
+    MOZ_ALWAYS_INLINE bool getForwardedPointer(T **ref);
 
     /* Forward a slots/elements pointer stored in an Ion frame. */
     void forwardBufferPointer(HeapSlot **pSlotsElems);
 
+    size_t sizeOfHeapCommitted() const {
+        return numActiveChunks_ * gc::ChunkSize;
+    }
+    size_t sizeOfHeapDecommitted() const {
+        return (numNurseryChunks_ - numActiveChunks_) * gc::ChunkSize;
+    }
+    size_t sizeOfHugeSlots(mozilla::MallocSizeOf mallocSizeOf) const {
+        size_t total = 0;
+        for (HugeSlotsSet::Range r = hugeSlots.all(); !r.empty(); r.popFront())
+            total += mallocSizeOf(r.front());
+        total += hugeSlots.sizeOfExcludingThis(mallocSizeOf);
+        return total;
+    }
+
+    MOZ_ALWAYS_INLINE uintptr_t start() const {
+        return heapStart_;
+    }
+
+    MOZ_ALWAYS_INLINE uintptr_t heapEnd() const {
+        return heapEnd_;
+    }
+
 #ifdef JS_GC_ZEAL
-    /*
-     * In debug and zeal builds, these bytes indicate the state of an unused
-     * segment of nursery-allocated memory.
-     */
-    static const uint8_t FreshNursery = 0x2a;
-    static const uint8_t SweptNursery = 0x2b;
-    static const uint8_t AllocatedThing = 0x2c;
-    void enterZealMode() { numActiveChunks_ = NumNurseryChunks; }
+    void enterZealMode();
+    void leaveZealMode();
 #endif
 
   private:
@@ -131,15 +170,25 @@ class Nursery
     /* Pointer to the first unallocated byte in the nursery. */
     uintptr_t position_;
 
+    /* Pointer to the logical start of the Nursery. */
+    uintptr_t currentStart_;
+
     /* Pointer to the last byte of space in the current chunk. */
     uintptr_t currentEnd_;
 
+    /* Pointer to first and last address of the total nursery allocation. */
+    uintptr_t heapStart_;
+    uintptr_t heapEnd_;
+
     /* The index of the chunk that is currently being allocated from. */
     int currentChunk_;
 
     /* The index after the last chunk that we will allocate from. */
     int numActiveChunks_;
 
+    /* Number of chunks allocated for the nursery. */
+    int numNurseryChunks_;
+
     /*
      * The set of externally malloced slots potentially kept live by objects
      * stored in the nursery. Any external slots that do not belong to a
@@ -149,52 +198,49 @@ class Nursery
     HugeSlotsSet hugeSlots;
 
     /* The maximum number of slots allowed to reside inline in the nursery. */
-    static const size_t MaxNurserySlots = 100;
+    static const size_t MaxNurserySlots = 128;
 
     /* The amount of space in the mapped nursery available to allocations. */
-    static const size_t NurseryChunkUsableSize = gc::ChunkSize - sizeof(JSRuntime *);
+    static const size_t NurseryChunkUsableSize = gc::ChunkSize - sizeof(gc::ChunkTrailer);
 
     struct NurseryChunkLayout {
         char data[NurseryChunkUsableSize];
-        JSRuntime *runtime;
+        gc::ChunkTrailer trailer;
         uintptr_t start() { return uintptr_t(&data); }
-        uintptr_t end() { return uintptr_t(&runtime); }
+        uintptr_t end() { return uintptr_t(&trailer); }
     };
+    static_assert(sizeof(NurseryChunkLayout) == gc::ChunkSize,
+                  "Nursery chunk size must match gc::Chunk size.");
     NurseryChunkLayout &chunk(int index) const {
-        JS_STATIC_ASSERT(sizeof(NurseryChunkLayout) == gc::ChunkSize);
-        JS_ASSERT(index < NumNurseryChunks);
+        JS_ASSERT(index < numNurseryChunks_);
         JS_ASSERT(start());
         return reinterpret_cast<NurseryChunkLayout *>(start())[index];
     }
 
-    JS_ALWAYS_INLINE uintptr_t start() const {
-        JS_ASSERT(runtime_);
-        return ((JS::shadow::Runtime *)runtime_)->gcNurseryStart_;
+    MOZ_ALWAYS_INLINE void initChunk(int chunkno) {
+        NurseryChunkLayout &c = chunk(chunkno);
+        c.trailer.storeBuffer = JS::shadow::Runtime::asShadowRuntime(runtime())->gcStoreBufferPtr();
+        c.trailer.location = gc::ChunkLocationNursery;
+        c.trailer.runtime = runtime();
     }
 
-    JS_ALWAYS_INLINE uintptr_t heapEnd() const {
-        JS_ASSERT(runtime_);
-        return ((JS::shadow::Runtime *)runtime_)->gcNurseryEnd_;
-    }
-
-    JS_ALWAYS_INLINE void setCurrentChunk(int chunkno) {
-        JS_ASSERT(chunkno < NumNurseryChunks);
+    MOZ_ALWAYS_INLINE void setCurrentChunk(int chunkno) {
+        JS_ASSERT(chunkno < numNurseryChunks_);
         JS_ASSERT(chunkno < numActiveChunks_);
         currentChunk_ = chunkno;
         position_ = chunk(chunkno).start();
         currentEnd_ = chunk(chunkno).end();
+        initChunk(chunkno);
     }
 
-    JS_ALWAYS_INLINE uintptr_t allocationEnd() const {
+    void updateDecommittedRegion();
+
+    MOZ_ALWAYS_INLINE uintptr_t allocationEnd() const {
         JS_ASSERT(numActiveChunks_ > 0);
         return chunk(numActiveChunks_ - 1).end();
     }
 
-    JS_ALWAYS_INLINE bool isFullyGrown() const {
-        return numActiveChunks_ == NumNurseryChunks;
-    }
-
-    JS_ALWAYS_INLINE uintptr_t currentEnd() const {
+    MOZ_ALWAYS_INLINE uintptr_t currentEnd() const {
         JS_ASSERT(runtime_);
         JS_ASSERT(currentEnd_ == chunk(currentChunk_).end());
         return currentEnd_;
@@ -215,31 +261,39 @@ class Nursery
     /* Allocates a new GC thing from the tenured generation during minor GC. */
     void *allocateFromTenured(JS::Zone *zone, gc::AllocKind thingKind);
 
+    struct TenureCountCache;
+
+    /* Common internal allocator function. */
+    void *allocate(size_t size);
+
     /*
      * Move the object at |src| in the Nursery to an already-allocated cell
      * |dst| in Tenured.
      */
-    void collectToFixedPoint(gc::MinorCollectionTracer *trc);
-    JS_ALWAYS_INLINE void traceObject(gc::MinorCollectionTracer *trc, JSObject *src);
-    JS_ALWAYS_INLINE void markSlots(gc::MinorCollectionTracer *trc, HeapSlot *vp, uint32_t nslots);
-    JS_ALWAYS_INLINE void markSlots(gc::MinorCollectionTracer *trc, HeapSlot *vp, HeapSlot *end);
-    JS_ALWAYS_INLINE void markSlot(gc::MinorCollectionTracer *trc, HeapSlot *slotp);
+    void collectToFixedPoint(gc::MinorCollectionTracer *trc, TenureCountCache &tenureCounts);
+    MOZ_ALWAYS_INLINE void traceObject(gc::MinorCollectionTracer *trc, JSObject *src);
+    MOZ_ALWAYS_INLINE void markSlots(gc::MinorCollectionTracer *trc, HeapSlot *vp, uint32_t nslots);
+    MOZ_ALWAYS_INLINE void markSlots(gc::MinorCollectionTracer *trc, HeapSlot *vp, HeapSlot *end);
+    MOZ_ALWAYS_INLINE void markSlot(gc::MinorCollectionTracer *trc, HeapSlot *slotp);
     void *moveToTenured(gc::MinorCollectionTracer *trc, JSObject *src);
     size_t moveObjectToTenured(JSObject *dst, JSObject *src, gc::AllocKind dstKind);
     size_t moveElementsToTenured(JSObject *dst, JSObject *src, gc::AllocKind dstKind);
     size_t moveSlotsToTenured(JSObject *dst, JSObject *src, gc::AllocKind dstKind);
+    void forwardTypedArrayPointers(JSObject *dst, JSObject *src);
 
     /* Handle relocation of slots/elements pointers stored in Ion frames. */
     void setSlotsForwardingPointer(HeapSlot *oldSlots, HeapSlot *newSlots, uint32_t nslots);
     void setElementsForwardingPointer(ObjectElements *oldHeader, ObjectElements *newHeader,
                                       uint32_t nelems);
 
+    /* Free malloced pointers owned by freed things in the nursery. */
+    void freeHugeSlots();
+
     /*
      * Frees all non-live nursery-allocated things at the end of a minor
-     * collection. This operation takes time proportional to the number of
-     * dead things.
+     * collection.
      */
-    void sweep(JSRuntime *rt);
+    void sweep();
 
     /* Change the allocable space provided by the nursery. */
     void growAllocableSpace();
@@ -248,10 +302,7 @@ class Nursery
     static void MinorGCCallback(JSTracer *trc, void **thingp, JSGCTraceKind kind);
 
     friend class gc::MinorCollectionTracer;
-    friend class jit::CodeGenerator;
     friend class jit::MacroAssembler;
-    friend class jit::ICStubCompiler;
-    friend class jit::BaselineCompiler;
 };
 
 } /* namespace js */
