@@ -35,6 +35,3 @@ if (!("verifypostbarriers" in this)) {
 if (!("gcPreserveCode" in this)) {
   gcPreserveCode = function() { }
 }
-
-if ("options" in this)
-    options("allow_xml");
