@@ -6,9 +6,10 @@
 
 #include "jit/IonOptimizationLevels.h"
 
-#include "jsanalyze.h"
 #include "jsscript.h"
 
+#include "jit/Ion.h"
+
 using namespace js;
 using namespace js::jit;
 
@@ -32,6 +33,7 @@ OptimizationInfo::initNormalOptimizationInfo()
     licm_ = true;
     uce_ = true;
     rangeAnalysis_ = true;
+    autoTruncate_ = true;
     registerAllocator_ = RegisterAllocator_LSRA;
 
     inlineMaxTotalBytecodeLength_ = 1000;
@@ -54,6 +56,8 @@ OptimizationInfo::initAsmjsOptimizationInfo()
     level_ = Optimization_AsmJS;
     edgeCaseAnalysis_ = false;
     eliminateRedundantChecks_ = false;
+    autoTruncate_ = false;
+    registerAllocator_ = RegisterAllocator_Backtracking;
 }
 
 uint32_t
@@ -76,7 +80,7 @@ OptimizationInfo::usesBeforeCompile(JSScript *script, jsbytecode *pc) const
     if (script->length() > MAX_MAIN_THREAD_SCRIPT_SIZE)
         minUses = minUses * (script->length() / (double) MAX_MAIN_THREAD_SCRIPT_SIZE);
 
-    uint32_t numLocalsAndArgs = analyze::TotalSlots(script);
+    uint32_t numLocalsAndArgs = NumLocalsAndArgs(script);
     if (numLocalsAndArgs > MAX_MAIN_THREAD_LOCALS_AND_ARGS)
         minUses = minUses * (numLocalsAndArgs / (double) MAX_MAIN_THREAD_LOCALS_AND_ARGS);
 
