@@ -185,7 +185,7 @@ JSONSpewer::beginFunction(JSScript *script)
 
     beginObject();
     if (script)
-        stringProperty("name", "%s:%d", script->filename(), script->lineno);
+        stringProperty("name", "%s:%d", script->filename(), script->lineno());
     else
         stringProperty("name", "asm.js compilation");
     beginListProperty("passes");
@@ -228,6 +228,8 @@ JSONSpewer::spewMResumePoint(MResumePoint *rp)
     for (MResumePoint *iter = rp; iter; iter = iter->caller()) {
         for (int i = iter->numOperands() - 1; i >= 0; i--)
             integerValue(iter->getOperand(i)->id());
+        if (iter->caller())
+            stringValue("|");
     }
     endList();
 
@@ -344,7 +346,7 @@ JSONSpewer::spewLIns(LInstruction *ins)
 
     property("opcode");
     fprintf(fp_, "\"");
-    ins->print(fp_);
+    ins->dump(fp_);
     fprintf(fp_, "\"");
 
     beginListProperty("defs");
