@@ -42,7 +42,7 @@ AllocationIntegrityState::record()
             virtualRegisters[vreg] = phi->getDef(0);
             if (!info.outputs.append(*phi->getDef(0)))
                 return false;
-            for (size_t k = 0; k < phi->numOperands(); k++) {
+            for (size_t k = 0, kend = phi->numOperands(); k < kend; k++) {
                 if (!info.inputs.append(*phi->getOperand(k)))
                     return false;
             }
@@ -162,8 +162,7 @@ AllocationIntegrityState::check(bool populateSafepoints)
                 checkIntegrity(block, *riter, vreg, **alloc, populateSafepoints);
 
                 while (!worklist.empty()) {
-                    IntegrityItem item = worklist.back();
-                    worklist.popBack();
+                    IntegrityItem item = worklist.popCopy();
                     checkIntegrity(item.block, *item.block->rbegin(), item.vreg, item.alloc, populateSafepoints);
                 }
             }
@@ -230,10 +229,10 @@ AllocationIntegrityState::checkIntegrity(LBlock *block, LInstruction *ins,
     // inputs as it is not guaranteed the register allocator filled in physical
     // allocations for the inputs and outputs of the phis.
     for (size_t i = 0; i < block->numPhis(); i++) {
-        InstructionInfo &info = blocks[block->mir()->id()].phis[i];
+        const InstructionInfo &info = blocks[block->mir()->id()].phis[i];
         LPhi *phi = block->getPhi(i);
         if (info.outputs[0].virtualRegister() == vreg) {
-            for (size_t j = 0; j < phi->numOperands(); j++) {
+            for (size_t j = 0, jend = phi->numOperands(); j < jend; j++) {
                 uint32_t newvreg = info.inputs[j].toUse()->virtualRegister();
                 LBlock *predecessor = graph.getBlock(block->mir()->getPredecessor(j)->id());
                 if (!addPredecessor(predecessor, newvreg, alloc))
@@ -245,7 +244,7 @@ AllocationIntegrityState::checkIntegrity(LBlock *block, LInstruction *ins,
 
     // No phi which defined the vreg we are tracking, follow back through all
     // predecessors with the existing vreg.
-    for (size_t i = 0; i < block->mir()->numPredecessors(); i++) {
+    for (size_t i = 0, iend = block->mir()->numPredecessors(); i < iend; i++) {
         LBlock *predecessor = graph.getBlock(block->mir()->getPredecessor(i)->id());
         if (!addPredecessor(predecessor, vreg, alloc))
             return false;
@@ -372,23 +371,31 @@ AllocationIntegrityState::dump()
         fprintf(stderr, "\n");
 
         for (size_t i = 0; i < block->numPhis(); i++) {
-            InstructionInfo &info = blocks[blockIndex].phis[i];
+            const InstructionInfo &info = blocks[blockIndex].phis[i];
             LPhi *phi = block->getPhi(i);
+            CodePosition output(phi->id(), CodePosition::OUTPUT);
 
-            fprintf(stderr, "Phi v%u <-", info.outputs[0].virtualRegister());
+            // Don't print the inputOf for phi nodes, since it's never used.
+            fprintf(stderr, "[,%u Phi [def v%u %s] <-",
+                    output.pos(),
+                    info.outputs[0].virtualRegister(),
+                    phi->getDef(0)->output()->toString());
             for (size_t j = 0; j < phi->numOperands(); j++)
                 fprintf(stderr, " %s", info.inputs[j].toString());
-            fprintf(stderr, "\n");
+            fprintf(stderr, "]\n");
         }
 
         for (LInstructionIterator iter = block->begin(); iter != block->end(); iter++) {
             LInstruction *ins = *iter;
-            InstructionInfo &info = instructions[ins->id()];
+            const InstructionInfo &info = instructions[ins->id()];
 
             CodePosition input(ins->id(), CodePosition::INPUT);
             CodePosition output(ins->id(), CodePosition::OUTPUT);
 
-            fprintf(stderr, "[%u,%u %s]", input.pos(), output.pos(), ins->opName());
+            fprintf(stderr, "[");
+            if (input != CodePosition::MIN)
+                fprintf(stderr, "%u,%u ", input.pos(), output.pos());
+            fprintf(stderr, "%s]", ins->opName());
 
             if (ins->isMoveGroup()) {
                 LMoveGroup *group = ins->toMoveGroup();
@@ -424,8 +431,6 @@ AllocationIntegrityState::dump()
         }
     }
 
-    fprintf(stderr, "\nIntermediate Allocations:\n\n");
-
     // Print discovered allocations at the ends of blocks, in the order they
     // were discovered.
 
@@ -437,10 +442,14 @@ AllocationIntegrityState::dump()
         seenOrdered[item.index] = item;
     }
 
-    for (size_t i = 0; i < seenOrdered.length(); i++) {
-        IntegrityItem item = seenOrdered[i];
-        fprintf(stderr, "block %u reg v%u alloc %s\n",
-               item.block->mir()->id(), item.vreg, item.alloc.toString());
+    if (!seenOrdered.empty()) {
+        fprintf(stderr, "\nIntermediate Allocations:\n\n");
+
+        for (size_t i = 0; i < seenOrdered.length(); i++) {
+            IntegrityItem item = seenOrdered[i];
+            fprintf(stderr, "block %u reg v%u alloc %s\n",
+                   item.block->mir()->id(), item.vreg, item.alloc.toString());
+        }
     }
 
     fprintf(stderr, "\n");
@@ -479,7 +488,7 @@ RegisterAllocator::getInputMoveGroup(uint32_t ins)
     if (data->inputMoves())
         return data->inputMoves();
 
-    LMoveGroup *moves = new LMoveGroup;
+    LMoveGroup *moves = LMoveGroup::New(alloc());
     data->setInputMoves(moves);
     data->block()->insertBefore(data->ins(), moves);
 
@@ -495,11 +504,11 @@ RegisterAllocator::getMoveGroupAfter(uint32_t ins)
     if (data->movesAfter())
         return data->movesAfter();
 
-    LMoveGroup *moves = new LMoveGroup;
+    LMoveGroup *moves = LMoveGroup::New(alloc());
     data->setMovesAfter(moves);
 
     if (data->ins()->isLabel())
-        data->block()->insertAfter(data->block()->getEntryMoveGroup(), moves);
+        data->block()->insertAfter(data->block()->getEntryMoveGroup(alloc()), moves);
     else
         data->block()->insertAfter(data->ins(), moves);
     return moves;
