@@ -9,12 +9,14 @@
 
 #include "mozilla/Attributes.h"
 
+#include "jspubtd.h"
+
 #include "ds/IdValuePair.h"
 #include "vm/String.h"
 
 namespace js {
 
-class MOZ_STACK_CLASS JSONParser : private AutoGCRooter
+class MOZ_STACK_CLASS JSONParser : private JS::AutoGCRooter
 {
   public:
     enum ErrorHandling { RaiseError, NoError };
@@ -23,8 +25,8 @@ class MOZ_STACK_CLASS JSONParser : private AutoGCRooter
     /* Data members */
 
     JSContext * const cx;
-    StableCharPtr current;
-    const StableCharPtr begin, end;
+    JS::ConstTwoByteChars current;
+    const JS::ConstTwoByteChars begin, end;
 
     Value v;
 
@@ -75,11 +77,11 @@ class MOZ_STACK_CLASS JSONParser : private AutoGCRooter
             return * static_cast<PropertyVector *>(vector);
         }
 
-        StackEntry(ElementVector *elements)
+        explicit StackEntry(ElementVector *elements)
           : state(FinishArrayElement), vector(elements)
         {}
 
-        StackEntry(PropertyVector *properties)
+        explicit StackEntry(PropertyVector *properties)
           : state(FinishObjectMember), vector(properties)
         {}
 
@@ -107,7 +109,7 @@ class MOZ_STACK_CLASS JSONParser : private AutoGCRooter
     /* Public API */
 
     /* Create a parser for the provided JSON data. */
-    JSONParser(JSContext *cx, JS::StableCharPtr data, size_t length,
+    JSONParser(JSContext *cx, JS::ConstTwoByteChars data, size_t length,
                ErrorHandling errorHandling = RaiseError)
       : AutoGCRooter(cx, JSONPARSER),
         cx(cx),
