@@ -84,6 +84,8 @@ class Test:
         self.jitflags = []     # jit flags to enable
         self.slow = False      # True means the test is slow-running
         self.allow_oom = False # True means that OOM is not considered a failure
+        self.allow_overrecursed = False # True means that hitting recursion the
+                                        # limits is not considered a failure.
         self.valgrind = False  # True means run under valgrind
         self.tz_pacific = False # True means force Pacific time for the test
         self.expect_error = '' # Errors to expect and consider passing
@@ -94,6 +96,7 @@ class Test:
         t.jitflags = self.jitflags[:]
         t.slow = self.slow
         t.allow_oom = self.allow_oom
+        t.allow_overrecursed = self.allow_overrecursed
         t.valgrind = self.valgrind
         t.tz_pacific = self.tz_pacific
         t.expect_error = self.expect_error
@@ -138,6 +141,8 @@ class Test:
                         test.slow = True
                     elif name == 'allow-oom':
                         test.allow_oom = True
+                    elif name == 'allow-overrecursed':
+                        test.allow_overrecursed = True
                     elif name == 'valgrind':
                         test.valgrind = options.valgrind
                     elif name == 'tz-pacific':
@@ -374,9 +379,23 @@ def check_output(out, err, rc, timed_out, test):
             return False
 
     if rc != test.expect_status:
+        # Tests which expect a timeout check for exit code 6.
+        # Sometimes 0 is returned on Windows for unknown reasons.
+        # See bug 899697.
+        if sys.platform in ['win32', 'cygwin'] and rc == 0:
+            return True
+
         # Allow a non-zero exit code if we want to allow OOM, but only if we
         # actually got OOM.
-        return test.allow_oom and 'out of memory' in err and 'Assertion failure' not in err
+        if test.allow_oom and 'out of memory' in err and 'Assertion failure' not in err:
+            return True
+
+        # Allow a non-zero exit code if we want to all too-much-recursion and
+        # the test actually over-recursed.
+        if test.allow_overrecursed and 'too much recursion' in err and 'Assertion failure' not in err:
+            return True
+
+        return False
 
     return True
 
@@ -458,7 +477,8 @@ def run_tests_parallel(tests, prefix, options):
     try:
         testcnt = 0
         # Initially start as many jobs as allowed to run parallel
-        for i in range(min(options.max_jobs,total_tests)):
+        # Always enqueue at least one to avoid a curious deadlock
+        for i in range(max(1, min(options.max_jobs, total_tests))):
             notify_queue.put(True)
 
         # For every item in the notify queue, start one new worker.
@@ -571,6 +591,16 @@ def print_test_summary(num_tests, failures, complete, doing, options):
 
 def process_test_results(results, num_tests, options):
     pb = NullProgressBar()
+    failures = []
+    timeouts = 0
+    complete = False
+    doing = 'before starting'
+
+    if num_tests == 0:
+        pb.finish(True)
+        complete = True
+        return print_test_summary(num_tests, failures, complete, doing, options)
+
     if not options.hide_progress and not options.show_cmd and ProgressBar.conservative_isatty():
         fmt = [
             {'value': 'PASS',    'color': 'green'},
@@ -580,10 +610,6 @@ def process_test_results(results, num_tests, options):
         ]
         pb = ProgressBar(num_tests, fmt)
 
-    failures = []
-    timeouts = 0
-    complete = False
-    doing = 'before starting'
     try:
         for i, res in enumerate(results):
             if options.show_output:
