@@ -1,7 +1,6 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sts=4 et sw=4 tw=99:
- *
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=8 sts=4 et sw=4 tw=99: */
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -30,7 +29,6 @@ XPCCallContext::XPCCallContext(XPCContext::LangType callerLanguage,
         mXPCContext(nullptr),
         mJSContext(cx),
         mCallerLanguage(callerLanguage),
-        mFlattenedJSObject(cx),
         mWrapper(nullptr),
         mTearOff(nullptr),
         mName(cx)
@@ -78,14 +76,10 @@ XPCCallContext::XPCCallContext(XPCContext::LangType callerLanguage,
         }
     }
     if (mWrapper) {
-        mFlattenedJSObject = mWrapper->GetFlatJSObject();
-
         if (mTearOff)
             mScriptableInfo = nullptr;
         else
             mScriptableInfo = mWrapper->GetScriptableInfo();
-    } else {
-        MOZ_ASSERT(!mFlattenedJSObject, "What object do we have?");
     }
 
     if (!JSID_IS_VOID(name))
@@ -246,9 +240,8 @@ XPCCallContext::~XPCCallContext()
 NS_IMETHODIMP
 XPCCallContext::GetCallee(nsISupports * *aCallee)
 {
-    nsISupports* temp = mWrapper ? mWrapper->GetIdentityObject() : nullptr;
-    NS_IF_ADDREF(temp);
-    *aCallee = temp;
+    nsCOMPtr<nsISupports> rval = mWrapper ? mWrapper->GetIdentityObject() : nullptr;
+    rval.forget(aCallee);
     return NS_OK;
 }
 
@@ -260,23 +253,12 @@ XPCCallContext::GetCalleeMethodIndex(uint16_t *aCalleeMethodIndex)
     return NS_OK;
 }
 
-/* readonly attribute nsIXPConnectWrappedNative CalleeWrapper; */
-NS_IMETHODIMP
-XPCCallContext::GetCalleeWrapper(nsIXPConnectWrappedNative * *aCalleeWrapper)
-{
-    nsIXPConnectWrappedNative* temp = mWrapper;
-    NS_IF_ADDREF(temp);
-    *aCalleeWrapper = temp;
-    return NS_OK;
-}
-
 /* readonly attribute XPCNativeInterface CalleeInterface; */
 NS_IMETHODIMP
 XPCCallContext::GetCalleeInterface(nsIInterfaceInfo * *aCalleeInterface)
 {
-    nsIInterfaceInfo* temp = mInterface->GetInterfaceInfo();
-    NS_IF_ADDREF(temp);
-    *aCalleeInterface = temp;
+    nsCOMPtr<nsIInterfaceInfo> rval = mInterface->GetInterfaceInfo();
+    rval.forget(aCalleeInterface);
     return NS_OK;
 }
 
@@ -284,9 +266,8 @@ XPCCallContext::GetCalleeInterface(nsIInterfaceInfo * *aCalleeInterface)
 NS_IMETHODIMP
 XPCCallContext::GetCalleeClassInfo(nsIClassInfo * *aCalleeClassInfo)
 {
-    nsIClassInfo* temp = mWrapper ? mWrapper->GetClassInfo() : nullptr;
-    NS_IF_ADDREF(temp);
-    *aCalleeClassInfo = temp;
+    nsCOMPtr<nsIClassInfo> rval = mWrapper ? mWrapper->GetClassInfo() : nullptr;
+    rval.forget(aCalleeClassInfo);
     return NS_OK;
 }
 
