@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=8 sts=4 et sw=4 tw=99: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -7,6 +8,8 @@
 #include "jsfriendapi.h"
 #include "jsprf.h"
 #include "nsCOMPtr.h"
+#include "AccessCheck.h"
+#include "WrapperFactory.h"
 #include "xpcprivate.h"
 #include "XPCInlines.h"
 #include "XPCQuickStubs.h"
@@ -120,10 +123,10 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *protoArg, unsigned flags,
                 for ( ; ps < ps_end; ++ps) {
                     if (!JS_DefineProperty(cx, proto,
                                            stringTable + ps->name_index,
-                                           JSVAL_VOID,
+                                           JS::UndefinedHandleValue,
+                                           flags | JSPROP_SHARED | JSPROP_NATIVE_ACCESSORS,
                                            (JSPropertyOp)ps->getter,
-                                           (JSStrictPropertyOp)ps->setter,
-                                           flags | JSPROP_SHARED | JSPROP_NATIVE_ACCESSORS))
+                                           (JSStrictPropertyOp)ps->setter))
                         return false;
                 }
 
@@ -139,7 +142,13 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *protoArg, unsigned flags,
                 }
 
                 if (entry->newBindingProperties) {
-                    mozilla::dom::DefineWebIDLBindingPropertiesOnXPCProto(cx, proto, entry->newBindingProperties);
+                    if (entry->newBindingProperties->regular) {
+                        mozilla::dom::DefineWebIDLBindingPropertiesOnXPCObject(cx, proto, entry->newBindingProperties->regular);
+                    }
+                    if (entry->newBindingProperties->chromeOnly &&
+                        xpc::AccessCheck::isChrome(js::GetContextCompartment(cx))) {
+                        mozilla::dom::DefineWebIDLBindingPropertiesOnXPCObject(cx, proto, entry->newBindingProperties->chromeOnly);
+                    }
                 }
                 // Next.
                 size_t j = entry->parentInterface;
@@ -197,12 +206,13 @@ GetMemberInfo(JSObject *obj, jsid memberId, const char **ifaceName)
 static void
 GetMethodInfo(JSContext *cx, jsval *vp, const char **ifaceNamep, jsid *memberIdp)
 {
-    RootedObject funobj(cx, JSVAL_TO_OBJECT(JS_CALLEE(cx, vp)));
+    CallReceiver call = CallReceiverFromVp(vp);
+    RootedObject funobj(cx, &call.callee());
     MOZ_ASSERT(JS_ObjectIsFunction(cx, funobj),
                "JSNative callee should be Function object");
     RootedString str(cx, JS_GetFunctionId(JS_GetObjectFunction(funobj)));
     RootedId methodId(cx, str ? INTERNED_STRING_TO_JSID(cx, str) : JSID_VOID);
-    GetMemberInfo(JSVAL_TO_OBJECT(vp[1]), methodId, ifaceNamep);
+    GetMemberInfo(&call.thisv().toObject(), methodId, ifaceNamep);
     *memberIdp = methodId;
 }
 
@@ -297,21 +307,6 @@ xpc_qsThrowMethodFailed(JSContext *cx, nsresult rv, jsval *vp)
     return ThrowCallFailed(cx, rv, ifaceName, memberId, nullptr);
 }
 
-bool
-xpc_qsThrowMethodFailedWithCcx(XPCCallContext &ccx, nsresult rv)
-{
-    ThrowBadResult(rv, ccx);
-    return false;
-}
-
-bool
-xpc_qsThrowMethodFailedWithDetails(JSContext *cx, nsresult rv,
-                                   const char *ifaceName,
-                                   const char *memberName)
-{
-    return ThrowCallFailed(cx, rv, ifaceName, JSID_VOIDHANDLE, memberName);
-}
-
 static void
 ThrowBadArg(JSContext *cx, nsresult rv, const char *ifaceName,
             jsid memberId, const char *memberName, unsigned paramnum)
@@ -470,14 +465,14 @@ xpc_qsACString::xpc_qsACString(JSContext *cx, HandleValue v,
 
 xpc_qsAUTF8String::xpc_qsAUTF8String(JSContext *cx, HandleValue v, MutableHandleValue pval, bool notpassed)
 {
-    typedef nsCharTraits<PRUnichar> traits;
+    typedef nsCharTraits<char16_t> traits;
     // From the T_UTF8STRING  case in XPCConvert::JSData2Native.
     JSString *s = InitOrStringify<traits>(cx, v, pval, notpassed, eNull, eNull);
     if (!s)
         return;
 
     size_t len;
-    const PRUnichar *chars = JS_GetStringCharsZAndLength(cx, s, &len);
+    const char16_t *chars = JS_GetStringCharsZAndLength(cx, s, &len);
     if (!chars) {
         mValid = false;
         return;
@@ -531,14 +526,28 @@ getWrapper(JSContext *cx,
     // If we pass stopAtOuter == false, we can handle all three with one call
     // to js::CheckedUnwrap.
     if (js::IsWrapper(obj)) {
-        obj = js::CheckedUnwrap(obj, /* stopAtOuter = */ false);
+        JSObject* inner = js::CheckedUnwrap(obj, /* stopAtOuter = */ false);
+
+        // Hack - For historical reasons, wrapped chrome JS objects have been
+        // passable as native interfaces. We'd like to fix this, but it
+        // involves fixing the contacts API and PeerConnection to stop using
+        // COWs. This needs to happen, but for now just preserve the old
+        // behavior.
+        //
+        // Note that there is an identical hack in
+        // XPCConvert::JSObject2NativeInterface which should be removed if this
+        // one is.
+        if (!inner && MOZ_UNLIKELY(xpc::WrapperFactory::IsCOW(obj)))
+            inner = js::UncheckedUnwrap(obj);
 
         // The safe unwrap might have failed if we encountered an object that
         // we're not allowed to unwrap. If it didn't fail though, we should be
         // done with wrappers.
-        if (!obj)
+        if (!inner)
             return NS_ERROR_XPC_SECURITY_MANAGER_VETO;
-        MOZ_ASSERT(!js::IsWrapper(obj));
+        MOZ_ASSERT(!js::IsWrapper(inner));
+
+        obj = inner;
     }
 
     // Start with sane values.
@@ -659,24 +668,6 @@ castNativeFromWrapper(JSContext *cx,
     return native;
 }
 
-bool
-xpc_qsUnwrapThisFromCcxImpl(XPCCallContext &ccx,
-                            const nsIID &iid,
-                            void **ppThis,
-                            nsISupports **pThisRef,
-                            jsval *vp)
-{
-    nsISupports *native = ccx.GetIdentityObject();
-    if (!native)
-        return xpc_qsThrow(ccx.GetJSContext(), NS_ERROR_XPC_HAS_BEEN_SHUTDOWN);
-
-    RootedObject obj(ccx, ccx.GetFlattenedJSObject());
-    nsresult rv = getNative(native, obj, iid, ppThis, pThisRef, vp);
-    if (NS_FAILED(rv))
-        return xpc_qsThrow(ccx.GetJSContext(), rv);
-    return true;
-}
-
 nsresult
 xpc_qsUnwrapArgImpl(JSContext *cx,
                     HandleValue v,
@@ -747,56 +738,28 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
 }
 
 bool
-xpc_qsJsvalToCharStr(JSContext *cx, jsval v, JSAutoByteString *bytes)
+xpc_qsJsvalToCharStr(JSContext *cx, HandleValue v, JSAutoByteString *bytes)
 {
-    JSString *str;
-
     MOZ_ASSERT(!bytes->ptr());
-    if (JSVAL_IS_STRING(v)) {
-        str = JSVAL_TO_STRING(v);
-    } else if (JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v)) {
-        return true;
-    } else {
-        if (!(str = JS_ValueToString(cx, v)))
-            return false;
-    }
+
+    if (v.isNullOrUndefined())
+      return true;
+
+    JSString *str = ToString(cx, v);
+    if (!str)
+      return false;
     return !!bytes->encodeLatin1(cx, str);
 }
 
-bool
-xpc_qsJsvalToWcharStr(JSContext *cx, jsval v, jsval *pval, const PRUnichar **pstr)
-{
-    JSString *str;
-
-    if (JSVAL_IS_STRING(v)) {
-        str = JSVAL_TO_STRING(v);
-    } else if (JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v)) {
-        *pstr = nullptr;
-        return true;
-    } else {
-        if (!(str = JS_ValueToString(cx, v)))
-            return false;
-        *pval = STRING_TO_JSVAL(str);  // Root the new string.
-    }
-
-    const jschar *chars = JS_GetStringCharsZ(cx, str);
-    if (!chars)
-        return false;
-
-    *pstr = static_cast<const PRUnichar *>(chars);
-    return true;
-}
-
 namespace xpc {
 
 bool
 NonVoidStringToJsval(JSContext *cx, nsAString &str, MutableHandleValue rval)
 {
     nsStringBuffer* sharedBuffer;
-    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer);
-    if (JSVAL_IS_NULL(jsstr))
-        return false;
-    rval.set(jsstr);
+    if (!XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer, rval))
+      return false;
+
     if (sharedBuffer) {
         // The string was shared but ReadableToJSVal didn't addref it.
         // Move the ownership from str to jsstr.
@@ -807,28 +770,6 @@ NonVoidStringToJsval(JSContext *cx, nsAString &str, MutableHandleValue rval)
 
 } // namespace xpc
 
-bool
-xpc_qsStringToJsstring(JSContext *cx, nsString &str, JSString **rval)
-{
-    // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
-    if (str.IsVoid()) {
-        *rval = nullptr;
-        return true;
-    }
-
-    nsStringBuffer* sharedBuffer;
-    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer);
-    if (JSVAL_IS_NULL(jsstr))
-        return false;
-    *rval = JSVAL_TO_STRING(jsstr);
-    if (sharedBuffer) {
-        // The string was shared but ReadableToJSVal didn't addref it.
-        // Move the ownership from str to jsstr.
-        str.ForgetSharedBuffer();
-    }
-    return true;
-}
-
 bool
 xpc_qsXPCOMObjectToJsval(JSContext *cx, qsObjectHelper &aHelper,
                          const nsIID *iid, XPCNativeInterface **iface,
