@@ -18,8 +18,10 @@
 #include "VideoUtils.h"
 #endif
 #include "MediaConduitInterface.h"
+#include "MediaPipelineFilter.h"
 #include "AudioSegment.h"
 #include "mozilla/ReentrantMonitor.h"
+#include "mozilla/Atomics.h"
 #include "SrtpFlow.h"
 #include "databuffer.h"
 #include "runnable_utils.h"
@@ -29,8 +31,12 @@
 #include "VideoSegment.h"
 #endif
 
+#include "webrtc/modules/rtp_rtcp/interface/rtp_header_parser.h"
+
 namespace mozilla {
 
+class PeerIdentity;
+
 // A class that represents the pipeline of audio and video
 // The dataflow looks like:
 //
@@ -80,16 +86,11 @@ class MediaPipeline : public sigslot::has_slots<> {
         track_id_(track_id),
         level_(level),
         conduit_(conduit),
-        rtp_transport_(rtp_transport),
-        rtp_state_(MP_CONNECTING),
-        rtcp_transport_(rtcp_transport),
-        rtcp_state_(MP_CONNECTING),
+        rtp_(rtp_transport, rtcp_transport ? RTP : MUX),
+        rtcp_(rtcp_transport ? rtcp_transport : rtp_transport,
+              rtcp_transport ? RTCP : MUX),
         main_thread_(main_thread),
         sts_thread_(sts_thread),
-        rtp_send_srtp_(),
-        rtcp_send_srtp_(),
-        rtp_recv_srtp_(),
-        rtcp_recv_srtp_(),
         rtp_packets_sent_(0),
         rtcp_packets_sent_(0),
         rtp_packets_received_(0),
@@ -101,11 +102,8 @@ class MediaPipeline : public sigslot::has_slots<> {
       // To indicate rtcp-mux rtcp_transport should be nullptr.
       // Therefore it's an error to send in the same flow for
       // both rtp and rtcp.
-      MOZ_ASSERT(rtp_transport_ != rtcp_transport_);
+      MOZ_ASSERT(rtp_transport != rtcp_transport);
 
-      if (!rtcp_transport_) {
-        rtcp_transport_ = rtp_transport;
-      }
       // PipelineTransport() will access this->sts_thread_; moved here for safety
       transport_ = new PipelineTransport(this);
   }
@@ -126,12 +124,22 @@ class MediaPipeline : public sigslot::has_slots<> {
 
   virtual nsresult Init();
 
+  // When we have offered bundle, the MediaPipelines are created in an
+  // indeterminate state; we do not know whether the answerer will take us
+  // up on our offer. In the meantime, we need to behave in a manner that
+  // errs on the side of packet loss when it is unclear whether an arriving
+  // packet is meant for us. We want to get out of this indeterminate state
+  // ASAP, which is what this function can be used for.
+  void SetUsingBundle_s(bool decision);
+  MediaPipelineFilter* UpdateFilterFromRemoteDescription_s(
+      nsAutoPtr<MediaPipelineFilter> filter);
+
   virtual Direction direction() const { return direction_; }
   virtual TrackID trackid() const { return track_id_; }
   virtual int level() const { return level_; }
 
   bool IsDoingRtcpMux() const {
-    return (rtp_transport_ == rtcp_transport_);
+    return (rtp_.type_ == MUX);
   }
 
   int32_t rtp_packets_sent() const { return rtp_packets_sent_; }
@@ -146,6 +154,13 @@ class MediaPipeline : public sigslot::has_slots<> {
   // Thread counting
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(MediaPipeline)
 
+  typedef enum {
+    RTP,
+    RTCP,
+    MUX,
+    MAX_RTP_TYPE
+  } RtpType;
+
  protected:
   virtual void DetachMediaStream() {}
 
@@ -153,7 +168,7 @@ class MediaPipeline : public sigslot::has_slots<> {
   class PipelineTransport : public TransportInterface {
    public:
     // Implement the TransportInterface functions
-    PipelineTransport(MediaPipeline *pipeline)
+    explicit PipelineTransport(MediaPipeline *pipeline)
         : pipeline_(pipeline),
           sts_thread_(pipeline->sts_thread_) {}
 
@@ -172,15 +187,40 @@ class MediaPipeline : public sigslot::has_slots<> {
   };
   friend class PipelineTransport;
 
-  virtual nsresult TransportFailed_s(TransportFlow *flow);  // The transport is down
-  virtual nsresult TransportReady_s(TransportFlow *flow);   // The transport is ready
+  class TransportInfo {
+    public:
+      TransportInfo(RefPtr<TransportFlow> flow, RtpType type) :
+        transport_(flow),
+        state_(MP_CONNECTING),
+        type_(type) {
+        MOZ_ASSERT(flow);
+      }
+
+      RefPtr<TransportFlow> transport_;
+      State state_;
+      RefPtr<SrtpFlow> send_srtp_;
+      RefPtr<SrtpFlow> recv_srtp_;
+      RtpType type_;
+  };
+
+  // The transport is down
+  virtual nsresult TransportFailed_s(TransportInfo &info);
+  // The transport is ready
+  virtual nsresult TransportReady_s(TransportInfo &info);
+  void UpdateRtcpMuxState(TransportInfo &info);
+
+  // Unhooks from signals
+  void DisconnectTransport_s(TransportInfo &info);
+  nsresult ConnectTransport_s(TransportInfo &info);
+
+  TransportInfo* GetTransportInfo_s(TransportFlow *flow);
 
   void increment_rtp_packets_sent(int bytes);
   void increment_rtcp_packets_sent();
   void increment_rtp_packets_received(int bytes);
   void increment_rtcp_packets_received();
 
-  virtual nsresult SendPacket(TransportFlow *flow, const void* data, int len);
+  virtual nsresult SendPacket(TransportFlow *flow, const void *data, int len);
 
   // Process slots on transports
   void StateChange(TransportFlow *flow, TransportLayer::State);
@@ -202,10 +242,18 @@ class MediaPipeline : public sigslot::has_slots<> {
                                          // thread. Read on STS thread.
 
   // The transport objects. Read/written on STS thread.
-  RefPtr<TransportFlow> rtp_transport_;
-  State rtp_state_;
-  RefPtr<TransportFlow> rtcp_transport_;
-  State rtcp_state_;
+  TransportInfo rtp_;
+  TransportInfo rtcp_;
+  // These are for bundle. We have a separate set because when we have offered
+  // bundle, we do not know whether we will receive traffic on the transport
+  // in this pipeline's m-line, or the transport in the "master" m-line for
+  // the bundle. We need to be ready for either. Once this ambiguity is
+  // resolved, the transport we know that we'll be using will be set in
+  // rtp_transport_ and rtcp_transport_, and these will be unset.
+  // TODO(bcampen@mozilla.com): I'm pretty sure this could be leveraged for
+  // re-offer with a new address on an m-line too, with a little work.
+  nsAutoPtr<TransportInfo> possible_bundle_rtp_;
+  nsAutoPtr<TransportInfo> possible_bundle_rtcp_;
 
   // Pointers to the threads we need. Initialized at creation
   // and used all over the place.
@@ -216,15 +264,8 @@ class MediaPipeline : public sigslot::has_slots<> {
   // destroyed on the STS thread.
   RefPtr<PipelineTransport> transport_;
 
-  // Used only on STS thread.
-  RefPtr<SrtpFlow> rtp_send_srtp_;
-  RefPtr<SrtpFlow> rtcp_send_srtp_;
-  RefPtr<SrtpFlow> rtp_recv_srtp_;
-  RefPtr<SrtpFlow> rtcp_recv_srtp_;
-
-  // Written only on STS thread. May be read on other
-  // threads but since there is no mutex, the values
-  // will only be approximate.
+  // Only safe to access from STS thread.
+  // Build into TransportInfo?
   int32_t rtp_packets_sent_;
   int32_t rtcp_packets_sent_;
   int32_t rtp_packets_received_;
@@ -236,6 +277,10 @@ class MediaPipeline : public sigslot::has_slots<> {
   std::string pc_;
   std::string description_;
 
+  // Written on Init, all following accesses are on the STS thread.
+  nsAutoPtr<MediaPipelineFilter> filter_;
+  nsAutoPtr<webrtc::RtpHeaderParser> rtp_parser_;
+
  private:
   nsresult Init_s();
 
@@ -312,7 +357,7 @@ private:
 // A specialization of pipeline for reading from an input device
 // and transmitting to the network.
 class MediaPipelineTransmit : public MediaPipeline {
- public:
+public:
   // Set rtcp_transport to nullptr to use rtcp-mux
   MediaPipelineTransmit(const std::string& pc,
                         nsCOMPtr<nsIEventTarget> main_thread,
@@ -331,7 +376,14 @@ class MediaPipelineTransmit : public MediaPipeline {
   {}
 
   // Initialize (stuff here may fail)
-  virtual nsresult Init();
+  virtual nsresult Init() MOZ_OVERRIDE;
+
+#ifdef MOZILLA_INTERNAL_API
+  // when the principal of the PeerConnection changes, it calls through to here
+  // so that we can determine whether to enable stream transmission
+  virtual void UpdateSinkIdentity_m(nsIPrincipal* principal,
+                                    const PeerIdentity* sinkIdentity);
+#endif
 
   // Called on the main thread.
   virtual void DetachMediaStream() {
@@ -344,7 +396,7 @@ class MediaPipelineTransmit : public MediaPipeline {
   }
 
   // Override MediaPipeline::TransportReady.
-  virtual nsresult TransportReady_s(TransportFlow *flow);
+  virtual nsresult TransportReady_s(TransportInfo &info);
 
   // Separate class to allow ref counting
   class PipelineListener : public MediaStreamDirectListener {
@@ -353,6 +405,7 @@ class MediaPipelineTransmit : public MediaPipeline {
     PipelineListener(const RefPtr<MediaSessionConduit>& conduit)
       : conduit_(conduit),
         active_(false),
+        enabled_(false),
         direct_connect_(false),
         samples_10ms_buffer_(nullptr),
         buffer_current_(0),
@@ -367,18 +420,15 @@ class MediaPipelineTransmit : public MediaPipeline {
     {
       // release conduit on mainthread.  Must use forget()!
       nsresult rv = NS_DispatchToMainThread(new
-        ConduitDeleteEvent(conduit_.forget()), NS_DISPATCH_NORMAL);
+        ConduitDeleteEvent(conduit_.forget()));
       MOZ_ASSERT(!NS_FAILED(rv),"Could not dispatch conduit shutdown to main");
       if (NS_FAILED(rv)) {
         MOZ_CRASH();
       }
     }
 
-
-    // XXX. This is not thread-safe but the hazard is just
-    // that active_ = true takes a while to propagate. Revisit
-    // when 823600 lands.
     void SetActive(bool active) { active_ = active; }
+    void SetEnabled(bool enabled) { enabled_ = enabled; }
 
     // Implement MediaStreamListener
     virtual void NotifyQueuedTrackChanges(MediaStreamGraph* graph, TrackID tid,
@@ -409,9 +459,16 @@ class MediaPipelineTransmit : public MediaPipeline {
                                    TrackRate rate, VideoChunk& chunk);
 #endif
     RefPtr<MediaSessionConduit> conduit_;
-    volatile bool active_;
+
+    // active is true if there is a transport to send on
+    mozilla::Atomic<bool> active_;
+    // enabled is true if the media access control permits sending
+    // actual content; when false you get black/silence
+    mozilla::Atomic<bool> enabled_;
+
     bool direct_connect_;
 
+
     // These vars handle breaking audio samples into exact 10ms chunks:
     // The buffer of 10ms audio samples that we will send once full
     // (can be carried over from one call to another).
@@ -445,11 +502,26 @@ class MediaPipelineReceive : public MediaPipeline {
                        int level,
                        RefPtr<MediaSessionConduit> conduit,
                        RefPtr<TransportFlow> rtp_transport,
-                       RefPtr<TransportFlow> rtcp_transport) :
+                       RefPtr<TransportFlow> rtcp_transport,
+                       RefPtr<TransportFlow> bundle_rtp_transport,
+                       RefPtr<TransportFlow> bundle_rtcp_transport,
+                       nsAutoPtr<MediaPipelineFilter> filter) :
       MediaPipeline(pc, RECEIVE, main_thread, sts_thread,
                     stream, track_id, level, conduit, rtp_transport,
                     rtcp_transport),
       segments_added_(0) {
+    filter_ = filter;
+    rtp_parser_ = webrtc::RtpHeaderParser::Create();
+    if (bundle_rtp_transport) {
+      if (bundle_rtcp_transport) {
+        MOZ_ASSERT(bundle_rtp_transport != bundle_rtcp_transport);
+        possible_bundle_rtp_ = new TransportInfo(bundle_rtp_transport, RTP);
+        possible_bundle_rtcp_ = new TransportInfo(bundle_rtcp_transport, RTCP);
+      } else {
+        possible_bundle_rtp_ = new TransportInfo(bundle_rtp_transport, MUX);
+        possible_bundle_rtcp_ = new TransportInfo(bundle_rtp_transport, MUX);
+      }
+    }
   }
 
   int segments_added() const { return segments_added_; }
@@ -473,10 +545,14 @@ class MediaPipelineReceiveAudio : public MediaPipelineReceive {
                             int level,
                             RefPtr<AudioSessionConduit> conduit,
                             RefPtr<TransportFlow> rtp_transport,
-                            RefPtr<TransportFlow> rtcp_transport) :
+                            RefPtr<TransportFlow> rtcp_transport,
+                            RefPtr<TransportFlow> bundle_rtp_transport,
+                            RefPtr<TransportFlow> bundle_rtcp_transport,
+                            nsAutoPtr<MediaPipelineFilter> filter) :
       MediaPipelineReceive(pc, main_thread, sts_thread,
                            stream, track_id, level, conduit, rtp_transport,
-                           rtcp_transport),
+                           rtcp_transport, bundle_rtp_transport,
+                           bundle_rtcp_transport, filter),
       listener_(new PipelineListener(stream->AsSourceStream(),
                                      track_id, conduit)) {
   }
@@ -488,7 +564,7 @@ class MediaPipelineReceiveAudio : public MediaPipelineReceive {
     stream_ = nullptr;
   }
 
-  virtual nsresult Init();
+  virtual nsresult Init() MOZ_OVERRIDE;
 
  private:
   // Separate class to allow ref counting
@@ -501,7 +577,7 @@ class MediaPipelineReceiveAudio : public MediaPipelineReceive {
     {
       // release conduit on mainthread.  Must use forget()!
       nsresult rv = NS_DispatchToMainThread(new
-        ConduitDeleteEvent(conduit_.forget()), NS_DISPATCH_NORMAL);
+        ConduitDeleteEvent(conduit_.forget()));
       MOZ_ASSERT(!NS_FAILED(rv),"Could not dispatch conduit shutdown to main");
       if (NS_FAILED(rv)) {
         MOZ_CRASH();
@@ -536,10 +612,14 @@ class MediaPipelineReceiveVideo : public MediaPipelineReceive {
                             int level,
                             RefPtr<VideoSessionConduit> conduit,
                             RefPtr<TransportFlow> rtp_transport,
-                            RefPtr<TransportFlow> rtcp_transport) :
+                            RefPtr<TransportFlow> rtcp_transport,
+                            RefPtr<TransportFlow> bundle_rtp_transport,
+                            RefPtr<TransportFlow> bundle_rtcp_transport,
+                            nsAutoPtr<MediaPipelineFilter> filter) :
       MediaPipelineReceive(pc, main_thread, sts_thread,
                            stream, track_id, level, conduit, rtp_transport,
-                           rtcp_transport),
+                           rtcp_transport, bundle_rtp_transport,
+                           bundle_rtcp_transport, filter),
       renderer_(new PipelineRenderer(MOZ_THIS_IN_INITIALIZER_LIST())),
       listener_(new PipelineListener(stream->AsSourceStream(), track_id)) {
   }
@@ -558,7 +638,7 @@ class MediaPipelineReceiveVideo : public MediaPipelineReceive {
     stream_ = nullptr;
   }
 
-  virtual nsresult Init();
+  virtual nsresult Init() MOZ_OVERRIDE;
 
  private:
   class PipelineRenderer : public VideoRenderer {
@@ -578,9 +658,11 @@ class MediaPipelineReceiveVideo : public MediaPipelineReceive {
     virtual void RenderVideoFrame(const unsigned char* buffer,
                                   unsigned int buffer_size,
                                   uint32_t time_stamp,
-                                  int64_t render_time) {
+                                  int64_t render_time,
+                                  const ImageHandle& handle) {
       pipeline_->listener_->RenderVideoFrame(buffer, buffer_size, time_stamp,
-                                            render_time);
+                                             render_time,
+                                             handle.GetImage());
     }
 
    private:
@@ -613,8 +695,8 @@ class MediaPipelineReceiveVideo : public MediaPipelineReceive {
     void RenderVideoFrame(const unsigned char* buffer,
                           unsigned int buffer_size,
                           uint32_t time_stamp,
-                          int64_t render_time);
-
+                          int64_t render_time,
+                          const RefPtr<layers::Image>& video_image);
 
    private:
     int width_;
