@@ -77,7 +77,7 @@ class Fake_MediaStream {
     mListeners.erase(aListener);
   }
 
-  virtual Fake_SourceMediaStream *AsSourceStream() { return NULL; }
+  virtual Fake_SourceMediaStream *AsSourceStream() { return nullptr; }
 
   virtual nsresult Start() { return NS_OK; }
   virtual nsresult Stop() { return NS_OK; }
@@ -90,7 +90,7 @@ class Fake_MediaStream {
  protected:
   std::set<Fake_MediaStreamListener *> mListeners;
   mozilla::Mutex mMutex;  // Lock to prevent the listener list from being modified while
-  		 	  // executing Periodic().
+                          // executing Periodic().
 };
 
 class Fake_MediaPeriodic : public nsITimerCallback {
@@ -99,7 +99,7 @@ Fake_MediaPeriodic(Fake_MediaStream *aStream) : mStream(aStream),
                                                 mCount(0) {}
   virtual ~Fake_MediaPeriodic() {}
   void Detach() {
-    mStream = NULL;
+    mStream = nullptr;
   }
 
   int GetTimesCalled() { return mCount; }
@@ -242,6 +242,16 @@ public:
   uint32_t GetHintContents() const { return mHintContents; }
   void SetHintContents(uint32_t aHintContents) { mHintContents = aHintContents; }
 
+  void SetTrackEnabled(mozilla::TrackID aTrackID, bool aEnabled) {}
+
+  class PrincipalChangeObserver
+  {
+  public:
+    virtual void PrincipalChanged(Fake_DOMMediaStream* aMediaStream) = 0;
+  };
+  void AddPrincipalChangeObserver(void* ignoredObserver) {}
+  void RemovePrincipalChangeObserver(void* ignoredObserver) {}
+
 private:
   nsRefPtr<Fake_MediaStream> mMediaStream;
 
