@@ -6,6 +6,7 @@ package org.mozilla.gecko;
 
 import org.mozilla.gecko.db.BrowserContract;
 import org.mozilla.gecko.db.BrowserDB;
+import org.mozilla.gecko.home.HomePanelsManager;
 import org.mozilla.gecko.mozglue.GeckoLoader;
 import org.mozilla.gecko.util.Clipboard;
 import org.mozilla.gecko.util.HardwareUtils;
@@ -13,17 +14,40 @@ import org.mozilla.gecko.util.ThreadUtils;
 
 import android.app.Application;
 import android.content.Context;
+import android.content.SharedPreferences;
 import android.content.res.Configuration;
 import android.util.Log;
 
-public class GeckoApplication extends Application {
+public class GeckoApplication extends Application 
+    implements ContextGetter {
     private static final String LOG_TAG = "GeckoApplication";
 
+    private static volatile GeckoApplication instance;
+
     private boolean mInBackground;
     private boolean mPausedGecko;
 
     private LightweightTheme mLightweightTheme;
 
+    public GeckoApplication() {
+        super();
+        instance = this;
+    }
+
+    public static GeckoApplication get() {
+        return instance;
+    }
+
+    @Override
+    public Context getContext() {
+        return this;
+    }
+
+    @Override
+    public SharedPreferences getSharedPreferences() {
+        return GeckoSharedPrefs.forApp(this);
+    }
+
     /**
      * We need to do locale work here, because we need to intercept
      * each hit to onConfigurationChanged.
@@ -43,9 +67,9 @@ public class GeckoApplication extends Application {
         // Otherwise, correct the locale. This catches some cases that GeckoApp
         // doesn't get a chance to.
         try {
-            LocaleManager.correctLocale(getResources(), config);
+            BrowserLocaleManager.getInstance().correctLocale(this, getResources(), config);
         } catch (IllegalStateException ex) {
-            // GeckoApp hasn't started, so we have no ContextGetter in LocaleManager.
+            // GeckoApp hasn't started, so we have no ContextGetter in BrowserLocaleManager.
             Log.w(LOG_TAG, "Couldn't correct locale.", ex);
         }
 
@@ -93,9 +117,13 @@ public class GeckoApplication extends Application {
 
     @Override
     public void onCreate() {
-        HardwareUtils.init(getApplicationContext());
-        Clipboard.init(getApplicationContext());
-        GeckoLoader.loadMozGlue();
+        final Context context = getApplicationContext();
+        HardwareUtils.init(context);
+        Clipboard.init(context);
+        FilePicker.init(context);
+        GeckoLoader.loadMozGlue(context);
+        HomePanelsManager.getInstance().init(context);
+
         super.onCreate();
     }
 
