@@ -10,10 +10,13 @@ import java.util.EnumSet;
 import java.util.List;
 
 import org.mozilla.gecko.R;
+import org.mozilla.gecko.Telemetry;
+import org.mozilla.gecko.TelemetryContract;
 import org.mozilla.gecko.animation.PropertyAnimator;
 import org.mozilla.gecko.animation.ViewHelper;
 import org.mozilla.gecko.home.HomeAdapter.OnAddPanelListener;
 import org.mozilla.gecko.home.HomeConfig.PanelConfig;
+import org.mozilla.gecko.util.ThreadUtils;
 
 import android.content.Context;
 import android.graphics.drawable.Drawable;
@@ -30,7 +33,6 @@ import android.view.View;
 import android.view.ViewGroup;
 
 public class HomePager extends ViewPager {
-
     private static final int LOADER_ID_CONFIG = 0;
 
     private final Context mContext;
@@ -50,6 +52,10 @@ public class HomePager extends ViewPager {
     // Cached original ViewPager background.
     private final Drawable mOriginalBackground;
 
+    // Telemetry session for current panel.
+    private TelemetryContract.Session mCurrentPanelSession;
+    private String mCurrentPanelSessionSuffix;
+
     // Current load state of HomePager.
     private LoadState mLoadState;
 
@@ -226,6 +232,7 @@ public class HomePager extends ViewPager {
                             PropertyAnimator.Property.ALPHA,
                             1.0f);
         }
+        Telemetry.startUISession(TelemetryContract.Session.HOME);
     }
 
     /**
@@ -235,6 +242,10 @@ public class HomePager extends ViewPager {
         mVisible = false;
         setAdapter(null);
         mLoadState = LoadState.UNLOADED;
+
+        // Stop UI Telemetry sessions.
+        stopCurrentPanelTelemetrySession();
+        Telemetry.stopUISession(TelemetryContract.Session.HOME);
     }
 
     /**
@@ -342,6 +353,12 @@ public class HomePager extends ViewPager {
 
         final HomeAdapter adapter = (HomeAdapter) getAdapter();
 
+        // Disable any fragment loading until we have the initial
+        // panel selection done. Store previous value to restore
+        // it if necessary once the UI is fully updated.
+        final boolean canLoadHint = adapter.getCanLoadHint();
+        adapter.setCanLoadHint(false);
+
         // Destroy any existing panels currently loaded
         // in the pager.
         setAdapter(null);
@@ -399,6 +416,20 @@ public class HomePager extends ViewPager {
                 setCurrentItem(mDefaultPageIndex, false);
             }
         }
+
+        // If the load hint was originally true, this means the pager
+        // is not animating and it's fine to restore the load hint back.
+        if (canLoadHint) {
+            // The selection is updated asynchronously so we need to post to
+            // UI thread to give the pager time to commit the new page selection
+            // internally and load the right initial panel.
+            ThreadUtils.getUiHandler().post(new Runnable() {
+                @Override
+                public void run() {
+                    adapter.setCanLoadHint(true);
+                }
+            });
+        }
     }
 
     public void setOnPanelChangeListener(OnPanelChangeListener listener) {
@@ -447,6 +478,10 @@ public class HomePager extends ViewPager {
             if (mHomeBanner != null) {
                 mHomeBanner.setActive(position == mDefaultPageIndex);
             }
+
+            // Start a UI telemetry session for the newly selected panel.
+            final String newPanelId = ((HomeAdapter) getAdapter()).getPanelIdAtPosition(position);
+            startNewPanelTelemetrySession(newPanelId);
         }
 
         @Override
@@ -463,4 +498,31 @@ public class HomePager extends ViewPager {
         @Override
         public void onPageScrollStateChanged(int state) { }
     }
+
+    /**
+     * Start UI telemetry session for the a panel.
+     * If there is currently a session open for a panel,
+     * it will be stopped before a new one is started.
+     *
+     * @param panelId of panel to start a session for
+     */
+    private void startNewPanelTelemetrySession(String panelId) {
+        // Stop the current panel's session if we have one.
+        stopCurrentPanelTelemetrySession();
+
+        mCurrentPanelSession = TelemetryContract.Session.HOME_PANEL;
+        mCurrentPanelSessionSuffix = panelId;
+        Telemetry.startUISession(mCurrentPanelSession, mCurrentPanelSessionSuffix);
+    }
+
+    /**
+     * Stop the current panel telemetry session if one exists.
+     */
+    private void stopCurrentPanelTelemetrySession() {
+        if (mCurrentPanelSession != null) {
+            Telemetry.stopUISession(mCurrentPanelSession, mCurrentPanelSessionSuffix);
+            mCurrentPanelSession = null;
+            mCurrentPanelSessionSuffix = null;
+        }
+    }
 }
