@@ -18,20 +18,21 @@ from distutils.version import StrictVersion
 
 from mozboot.base import BaseBootstrapper
 
-HOMEBREW_BOOTSTRAP = 'http://raw.github.com/mxcl/homebrew/go'
+HOMEBREW_BOOTSTRAP = 'https://raw.github.com/Homebrew/homebrew/go/install'
 XCODE_APP_STORE = 'macappstore://itunes.apple.com/app/id497799835?mt=12'
 XCODE_LEGACY = 'https://developer.apple.com/downloads/download.action?path=Developer_Tools/xcode_3.2.6_and_ios_sdk_4.3__final/xcode_3.2.6_and_ios_sdk_4.3.dmg'
 HOMEBREW_AUTOCONF213 = 'https://raw.github.com/Homebrew/homebrew-versions/master/autoconf213.rb'
 
-MACPORTS_URL = {'8': 'https://distfiles.macports.org/MacPorts/MacPorts-2.1.3-10.8-MountainLion.pkg',
+MACPORTS_URL = {'9': 'https://distfiles.macports.org/MacPorts/MacPorts-2.2.1-10.9-Mavericks.pkg',
+                '8': 'https://distfiles.macports.org/MacPorts/MacPorts-2.1.3-10.8-MountainLion.pkg',
                 '7': 'https://distfiles.macports.org/MacPorts/MacPorts-2.1.3-10.7-Lion.pkg',
                 '6': 'https://distfiles.macports.org/MacPorts/MacPorts-2.1.3-10.6-SnowLeopard.pkg',}
 
-MACPORTS_CLANG_PACKAGE = 'clang-3.2'
+MACPORTS_CLANG_PACKAGE = 'clang-3.3'
 
 RE_CLANG_VERSION = re.compile('Apple (?:clang|LLVM) version (\d+\.\d+)')
 
-APPLE_CLANG_MINIMUM_VERSION = StrictVersion('4.0')
+APPLE_CLANG_MINIMUM_VERSION = StrictVersion('4.2')
 
 XCODE_REQUIRED = '''
 Xcode is required to build Firefox. Please complete the install of Xcode
@@ -52,6 +53,14 @@ You will need to download and install Xcode to build Firefox.
 Please complete the Xcode download and then relaunch this script.
 '''
 
+XCODE_NO_DEVELOPER_DIRECTORY = '''
+xcode-select says you don't have a developer directory configured. We think
+this is due to you not having Xcode installed (properly). We're going to
+attempt to install Xcode through the App Store. If the App Store thinks you
+have Xcode installed, please run xcode-select by hand until it stops
+complaining and then re-run this script.
+'''
+
 XCODE_COMMAND_LINE_TOOLS_MISSING = '''
 The Xcode command line tools are required to build Firefox.
 '''
@@ -142,10 +151,13 @@ before the path to your package manager's binaries (%s).
 This means that the package manager's binaries likely won't be
 detected properly.
 
-Please modify your shell's configuration (e.g. ~/.bash_profile) to
-have %s appear in $PATH before %s. e.g.
+Modify your shell's configuration (e.g. ~/.profile or
+~/.bash_profile) to have %s appear in $PATH before %s. e.g.
 
     export PATH=%s:$PATH
+
+Once this is done, start a new shell (likely Command+T) and run
+this bootstrap again.
 '''
 
 
@@ -182,18 +194,24 @@ class OSXBootstrapper(BaseBootstrapper):
         # paths like /Applications/Xcode5-DP6.app.
         elif self.os_version >= StrictVersion('10.7'):
             select = self.which('xcode-select')
-            output = self.check_output([select, '--print-path'])
+            try:
+                output = self.check_output([select, '--print-path'],
+                    stderr=subprocess.STDOUT)
+            except subprocess.CalledProcessError as e:
+                # This seems to appear on fresh OS X machines before any Xcode
+                # has been installed. It may only occur on OS X 10.9 and later.
+                if 'unable to get active developer directory' in e.output:
+                    print(XCODE_NO_DEVELOPER_DIRECTORY)
+                    self._install_xcode_app_store()
+                    assert False # Above should exit.
 
             # This isn't the most robust check in the world. It relies on the
             # default value not being in an application bundle, which seems to
             # hold on at least Mavericks.
             if '.app/' not in output:
                 print(XCODE_REQUIRED)
-
-                subprocess.check_call(['open', XCODE_APP_STORE])
-
-                print('Once the install has finished, please relaunch this script.')
-                sys.exit(1)
+                self._install_xcode_app_store()
+                assert False # Above should exit.
 
         # Once Xcode is installed, you need to agree to the license before you can
         # use it.
@@ -203,7 +221,12 @@ class OSXBootstrapper(BaseBootstrapper):
         except subprocess.CalledProcessError as e:
             if 'license' in e.output:
                 xcodebuild = self.which('xcodebuild')
-                subprocess.check_call([xcodebuild, '-license'])
+                try:
+                    subprocess.check_call([xcodebuild, '-license'],
+                        stderr=subprocess.STDOUT)
+                except subprocess.CalledProcessError as e:
+                    if 'requires admin privileges' in e.output:
+                        self.run_as_root([xcodebuild, '-license'])
 
         # Even then we're not done! We need to install the Xcode command line tools.
         # As of Mountain Lion, apparently the only way to do this is to go through a
@@ -226,6 +249,11 @@ class OSXBootstrapper(BaseBootstrapper):
                 print(INSTALL_XCODE_COMMAND_LINE_TOOLS_STEPS)
                 sys.exit(1)
 
+    def _install_xcode_app_store(self):
+        subprocess.check_call(['open', XCODE_APP_STORE])
+        print('Once the install has finished, please relaunch this script.')
+        sys.exit(1)
+
     def ensure_homebrew_packages(self):
         self.brew = self.which('brew')
         assert self.brew is not None
@@ -286,9 +314,9 @@ class OSXBootstrapper(BaseBootstrapper):
         if self.os_version < StrictVersion('10.7') and MACPORTS_CLANG_PACKAGE not in installed:
             print(PACKAGE_MANAGER_OLD_CLANG % ('MacPorts',))
             self.run_as_root([self.port, '-v', 'install', MACPORTS_CLANG_PACKAGE])
+            self.run_as_root([self.port, 'select', '--set', 'clang', 'mp-' + MACPORTS_CLANG_PACKAGE])
 
         self.run_as_root([self.port, 'select', '--set', 'python', 'python27'])
-        self.run_as_root([self.port, 'select', '--set', 'clang', 'mp-' + MACPORTS_CLANG_PACKAGE])
 
     def ensure_package_manager(self):
         '''
