@@ -30,13 +30,13 @@ class nsNSSSocketInfo : public mozilla::psm::TransportSecurityInfo,
 {
 public:
   nsNSSSocketInfo(mozilla::psm::SharedSSLState& aState, uint32_t providerFlags);
-  
+
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSISSLSOCKETCONTROL
   NS_DECL_NSICLIENTAUTHUSERDECISION
- 
-  nsresult SetForSTARTTLS(bool aForSTARTTLS);
-  nsresult GetForSTARTTLS(bool *aForSTARTTLS);
+
+  void SetForSTARTTLS(bool aForSTARTTLS);
+  bool GetForSTARTTLS();
 
   nsresult GetFileDescPtr(PRFileDesc** aFilePtr);
   nsresult SetFileDescPtr(PRFileDesc* aFilePtr);
@@ -45,20 +45,22 @@ public:
   void SetHandshakeNotPending() { mHandshakePending = false; }
 
   void GetPreviousCert(nsIX509Cert** _result);
-  
-  void SetHasCleartextPhase(bool aHasCleartextPhase);
-  bool GetHasCleartextPhase();
-  
+
   void SetTLSVersionRange(SSLVersionRange range) { mTLSVersionRange = range; }
   SSLVersionRange GetTLSVersionRange() const { return mTLSVersionRange; };
 
   PRStatus CloseSocketAndDestroy(
-                const nsNSSShutDownPreventionLock & proofOfLock);
-  
-  void SetNegotiatedNPN(const char *value, uint32_t length);
-  void SetHandshakeCompleted(bool aResumedSession);
+                const nsNSSShutDownPreventionLock& proofOfLock);
+
+  void SetNegotiatedNPN(const char* value, uint32_t length);
+
+  void SetHandshakeCompleted();
   void NoteTimeUntilReady();
 
+
+  void SetFalseStartCallbackCalled() { mFalseStartCallbackCalled = true; }
+  void SetFalseStarted() { mFalseStarted = true; }
+
   // Note that this is only valid *during* a handshake; at the end of the handshake,
   // it gets reset back to false.
   void SetFullHandshake() { mIsFullHandshake = true; }
@@ -82,7 +84,7 @@ public:
   // ignored.
   void SetCertVerificationResult(PRErrorCode errorCode,
               ::mozilla::psm::SSLErrorMessageType errorMessageType);
-  
+
   // for logging only
   PRBool IsWaitingForCertVerification() const
   {
@@ -101,16 +103,12 @@ public:
     MOZ_ASSERT(NS_SUCCEEDED(rv));
     return result;
   }
-  void SetSymmetricCipherUsed(uint16_t symmetricCipher)
+
+  void SetKEAKeyBits(uint32_t keaBits) { mKEAKeyBits = keaBits; }
+
+  void SetSSLVersionUsed(int16_t version)
   {
-    mSymmetricCipherUsed = symmetricCipher;
-  }
-  inline int16_t GetSymmetricCipherExpected() // infallible in nsISSLSocketControl
-  {
-    int16_t result;
-    mozilla::DebugOnly<nsresult> rv = GetSymmetricCipherExpected(&result);
-    MOZ_ASSERT(NS_SUCCEEDED(rv));
-    return result;
+    mSSLVersionUsed = version;
   }
 
 private:
@@ -122,28 +120,27 @@ private:
   bool mForSTARTTLS;
   SSLVersionRange mTLSVersionRange;
   bool mHandshakePending;
-  bool mHasCleartextPhase;
   bool mRememberClientAuthCertificate;
   bool mPreliminaryHandshakeDone; // after false start items are complete
-  PRIntervalTime mHandshakeStartTime;
-  bool mFirstServerHelloReceived;
 
   nsresult ActivateSSL();
 
   nsCString mNegotiatedNPN;
   bool      mNPNCompleted;
+  bool      mFalseStartCallbackCalled;
+  bool      mFalseStarted;
   bool      mIsFullHandshake;
   bool      mHandshakeCompleted;
   bool      mJoined;
   bool      mSentClientCert;
   bool      mNotedTimeUntilReady;
 
-  // mKEA* and mSymmetricCipher* are used in false start detetermination
-  // values are from nsISSLSocketControl
+  // mKEA* are used in false start and http/2 detetermination
+  // Values are from nsISSLSocketControl
   int16_t mKEAUsed;
   int16_t mKEAExpected;
-  int16_t mSymmetricCipherUsed;
-  int16_t mSymmetricCipherExpected;
+  uint32_t mKEAKeyBits;
+  int16_t mSSLVersionUsed;
 
   uint32_t mProviderFlags;
   mozilla::TimeStamp mSocketCreationTimestamp;
@@ -165,7 +162,7 @@ public:
   static PRIOMethods nsSSLIOLayerMethods;
   static PRIOMethods nsSSLPlaintextLayerMethods;
 
-  nsTHashtable<nsCStringHashKey> *mRenegoUnrestrictedSites;
+  nsTHashtable<nsCStringHashKey>* mRenegoUnrestrictedSites;
   bool mTreatUnsafeNegotiationAsBroken;
   int32_t mWarnLevelMissingRFC5746;
 
@@ -187,15 +184,15 @@ private:
   };
   nsDataHashtable<nsCStringHashKey, IntoleranceEntry> mTLSIntoleranceInfo;
 public:
-  void rememberTolerantAtVersion(const nsACString & hostname, int16_t port,
+  void rememberTolerantAtVersion(const nsACString& hostname, int16_t port,
                                  uint16_t tolerant);
-  bool rememberIntolerantAtVersion(const nsACString & hostname, int16_t port,
+  bool rememberIntolerantAtVersion(const nsACString& hostname, int16_t port,
                                    uint16_t intolerant, uint16_t minVersion);
-  void adjustForTLSIntolerance(const nsACString & hostname, int16_t port,
-                               /*in/out*/ SSLVersionRange & range);
+  void adjustForTLSIntolerance(const nsACString& hostname, int16_t port,
+                               /*in/out*/ SSLVersionRange& range);
 
-  void setRenegoUnrestrictedSites(const nsCString &str);
-  bool isRenegoUnrestrictedSite(const nsCString &str);
+  void setRenegoUnrestrictedSites(const nsCString& str);
+  bool isRenegoUnrestrictedSite(const nsCString& str);
   void clearStoredData();
 
   bool mFalseStartRequireNPN;
@@ -206,26 +203,26 @@ private:
 };
 
 nsresult nsSSLIOLayerNewSocket(int32_t family,
-                               const char *host,
+                               const char* host,
                                int32_t port,
-                               const char *proxyHost,
+                               const char* proxyHost,
                                int32_t proxyPort,
-                               PRFileDesc **fd,
-                               nsISupports **securityInfo,
+                               PRFileDesc** fd,
+                               nsISupports** securityInfo,
                                bool forSTARTTLS,
                                uint32_t flags);
 
 nsresult nsSSLIOLayerAddToSocket(int32_t family,
-                                 const char *host,
+                                 const char* host,
                                  int32_t port,
-                                 const char *proxyHost,
+                                 const char* proxyHost,
                                  int32_t proxyPort,
-                                 PRFileDesc *fd,
-                                 nsISupports **securityInfo,
+                                 PRFileDesc* fd,
+                                 nsISupports** securityInfo,
                                  bool forSTARTTLS,
                                  uint32_t flags);
 
 nsresult nsSSLIOLayerFreeTLSIntolerantSites();
-nsresult displayUnknownCertErrorAlert(nsNSSSocketInfo *infoObject, int error);
+nsresult displayUnknownCertErrorAlert(nsNSSSocketInfo* infoObject, int error);
 
 #endif /* _NSNSSIOLAYER_H */
