@@ -184,6 +184,10 @@ this.BrowserIDManager.prototype = {
 
     // Reset the world before we do anything async.
     this.whenReadyToAuthenticate = Promise.defer();
+    this.whenReadyToAuthenticate.promise.then(null, (err) => {
+      this._log.error("Could not authenticate: " + err);
+    });
+
     this._shouldHaveSyncKeyBundle = false;
     this._authFailureReason = null;
 
@@ -437,6 +441,9 @@ this.BrowserIDManager.prototype = {
   // Refresh the sync token for our user.
   _fetchTokenForUser: function() {
     let tokenServerURI = Svc.Prefs.get("tokenServerURI");
+    if (tokenServerURI.endsWith("/")) { // trailing slashes cause problems...
+      tokenServerURI = tokenServerURI.slice(0, -1);
+    }
     let log = this._log;
     let client = this._tokenServerClient;
     let fxa = this._fxaService;
