@@ -24,6 +24,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/LateWriteChecks.h"
 #include "mozIStorageCompletionCallback.h"
+#include "mozIStoragePendingStatement.h"
 
 #include "sqlite3.h"
 
@@ -93,28 +94,24 @@ StorageSQLiteDistinguishedAmount()
 nsresult
 ReportConn(nsIHandleReportCallback *aHandleReport,
            nsISupports *aData,
-           sqlite3 *aConn,
+           Connection *aConn,
            const nsACString &aPathHead,
            const nsACString &aKind,
            const nsACString &aDesc,
-           int aOption,
+           int32_t aOption,
            size_t *aTotal)
 {
   nsCString path(aPathHead);
   path.Append(aKind);
   path.AppendLiteral("-used");
 
-  int curr = 0, max = 0;
-  int rc = ::sqlite3_db_status(aConn, aOption, &curr, &max, 0);
-  nsresult rv = convertResultCode(rc);
+  int32_t val = aConn->getSqliteRuntimeStatus(aOption);
+  nsresult rv = aHandleReport->Callback(EmptyCString(), path,
+                                        nsIMemoryReporter::KIND_HEAP,
+                                        nsIMemoryReporter::UNITS_BYTES,
+                                        int64_t(val), aDesc, aData);
   NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = aHandleReport->Callback(EmptyCString(), path,
-                               nsIMemoryReporter::KIND_HEAP,
-                               nsIMemoryReporter::UNITS_BYTES, int64_t(curr),
-                               aDesc, aData);
-  NS_ENSURE_SUCCESS(rv, rv);
-  *aTotal += curr;
+  *aTotal += val;
 
   return NS_OK;
 }
@@ -147,14 +144,14 @@ Service::CollectReports(nsIHandleReportCallback *aHandleReport,
 
       nsCString pathHead("explicit/storage/sqlite/");
       pathHead.Append(conn->getFilename());
-      pathHead.AppendLiteral("/");
+      pathHead.Append('/');
 
       SQLiteMutexAutoLock lockedScope(conn->sharedDBMutex);
 
       NS_NAMED_LITERAL_CSTRING(stmtDesc,
         "Memory (approximate) used by all prepared statements used by "
         "connections to this database.");
-      rv = ReportConn(aHandleReport, aData, *conn.get(), pathHead,
+      rv = ReportConn(aHandleReport, aData, conn, pathHead,
                       NS_LITERAL_CSTRING("stmt"), stmtDesc,
                       SQLITE_DBSTATUS_STMT_USED, &totalConnSize);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -162,7 +159,7 @@ Service::CollectReports(nsIHandleReportCallback *aHandleReport,
       NS_NAMED_LITERAL_CSTRING(cacheDesc,
         "Memory (approximate) used by all pager caches used by connections "
         "to this database.");
-      rv = ReportConn(aHandleReport, aData, *conn.get(), pathHead,
+      rv = ReportConn(aHandleReport, aData, conn, pathHead,
                       NS_LITERAL_CSTRING("cache"), cacheDesc,
                       SQLITE_DBSTATUS_CACHE_USED, &totalConnSize);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -170,7 +167,7 @@ Service::CollectReports(nsIHandleReportCallback *aHandleReport,
       NS_NAMED_LITERAL_CSTRING(schemaDesc,
         "Memory (approximate) used to store the schema for all databases "
         "associated with connections to this database.");
-      rv = ReportConn(aHandleReport, aData, *conn.get(), pathHead,
+      rv = ReportConn(aHandleReport, aData, conn, pathHead,
                       NS_LITERAL_CSTRING("schema"), schemaDesc,
                       SQLITE_DBSTATUS_SCHEMA_USED, &totalConnSize);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -200,7 +197,7 @@ Service::CollectReports(nsIHandleReportCallback *aHandleReport,
 ////////////////////////////////////////////////////////////////////////////////
 //// Service
 
-NS_IMPL_ISUPPORTS3(
+NS_IMPL_ISUPPORTS(
   Service,
   mozIStorageService,
   nsIObserver,
@@ -341,6 +338,34 @@ Service::getConnections(/* inout */ nsTArray<nsRefPtr<Connection> >& aConnection
   aConnections.AppendElements(mConnections);
 }
 
+void
+Service::minimizeMemory()
+{
+  nsTArray<nsRefPtr<Connection> > connections;
+  getConnections(connections);
+
+  for (uint32_t i = 0; i < connections.Length(); i++) {
+    nsRefPtr<Connection> conn = connections[i];
+    if (conn->connectionReady()) {
+      NS_NAMED_LITERAL_CSTRING(shrinkPragma, "PRAGMA shrink_memory");
+      nsCOMPtr<mozIStorageConnection> syncConn = do_QueryInterface(
+        NS_ISUPPORTS_CAST(mozIStorageAsyncConnection*, conn));
+      DebugOnly<nsresult> rv;
+
+      if (!syncConn) {
+        nsCOMPtr<mozIStoragePendingStatement> ps;
+        rv = connections[i]->ExecuteSimpleSQLAsync(shrinkPragma, nullptr,
+          getter_AddRefs(ps));
+      } else {
+        rv = connections[i]->ExecuteSimpleSQL(shrinkPragma);
+      }
+
+      MOZ_ASSERT(NS_SUCCEEDED(rv),
+        "Should have been able to purge sqlite caches");
+    }
+  }
+}
+
 void
 Service::shutdown()
 {
@@ -464,6 +489,12 @@ const sqlite3_mem_methods memMethods = {
 
 #endif  // MOZ_STORAGE_MEMORY
 
+static const char* sObserverTopics[] = {
+  "memory-pressure",
+  "xpcom-shutdown",
+  "xpcom-shutdown-threads"
+};
+
 nsresult
 Service::initialize()
 {
@@ -497,10 +528,13 @@ Service::initialize()
   // observer service can only be used on the main thread.
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   NS_ENSURE_TRUE(os, NS_ERROR_FAILURE);
-  nsresult rv = os->AddObserver(this, "xpcom-shutdown", false);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = os->AddObserver(this, "xpcom-shutdown-threads", false);
-  NS_ENSURE_SUCCESS(rv, rv);
+
+  for (size_t i = 0; i < ArrayLength(sObserverTopics); ++i) {
+    nsresult rv = os->AddObserver(this, sObserverTopics[i], false);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+  }
 
   // We cache XPConnect for our language helpers.  XPConnect can only be
   // used on the main thread.
@@ -861,12 +895,18 @@ Service::BackupDatabaseFile(nsIFile *aDBFile,
 NS_IMETHODIMP
 Service::Observe(nsISupports *, const char *aTopic, const char16_t *)
 {
-  if (strcmp(aTopic, "xpcom-shutdown") == 0)
+  if (strcmp(aTopic, "memory-pressure") == 0) {
+    minimizeMemory();
+  } else if (strcmp(aTopic, "xpcom-shutdown") == 0) {
     shutdown();
-  if (strcmp(aTopic, "xpcom-shutdown-threads") == 0) {
+  } else if (strcmp(aTopic, "xpcom-shutdown-threads") == 0) {
     nsCOMPtr<nsIObserverService> os =
       mozilla::services::GetObserverService();
-    os->RemoveObserver(this, "xpcom-shutdown-threads");
+
+    for (size_t i = 0; i < ArrayLength(sObserverTopics); ++i) {
+      (void)os->RemoveObserver(this, sObserverTopics[i]);
+    }
+
     bool anyOpen = false;
     do {
       nsTArray<nsRefPtr<Connection> > connections;
@@ -874,9 +914,6 @@ Service::Observe(nsISupports *, const char *aTopic, const char16_t *)
       anyOpen = false;
       for (uint32_t i = 0; i < connections.Length(); i++) {
         nsRefPtr<Connection> &conn = connections[i];
-
-        // While it would be nice to close all connections, we only
-        // check async ones for now.
         if (conn->isClosing()) {
           anyOpen = true;
           break;
@@ -892,7 +929,7 @@ Service::Observe(nsISupports *, const char *aTopic, const char16_t *)
       nsTArray<nsRefPtr<Connection> > connections;
       getConnections(connections);
       for (uint32_t i = 0, n = connections.Length(); i < n; i++) {
-        if (connections[i]->ConnectionReady()) {
+        if (!connections[i]->isClosed()) {
           MOZ_CRASH();
         }
       }
