@@ -46,7 +46,7 @@
 #endif // defined (XP_UNIX)
 
 #if defined(XP_WIN)
-#include <Windows.h>
+#include <windows.h>
 #endif // defined (XP_WIN)
 
 namespace mozilla {
@@ -64,10 +64,6 @@ namespace {
  * and into JavaScript without copy.
  */
 struct ArrayBufferContents {
-  /**
-   * The header of the ArrayBuffer. This is the pointer actually used by JSAPI.
-   */
-  void* header;
   /**
    * The data of the ArrayBuffer. This is the pointer manipulated to
    * read/write the contents of the buffer.
@@ -85,12 +81,11 @@ struct ArrayBufferContents {
 struct ScopedArrayBufferContentsTraits {
   typedef ArrayBufferContents type;
   const static type empty() {
-    type result = {0, 0, 0};
+    type result = {0, 0};
     return result;
   }
   const static void release(type ptr) {
-    js_free(ptr.header);
-    ptr.header = nullptr;
+    js_free(ptr.data);
     ptr.data = nullptr;
     ptr.nbytes = 0;
   }
@@ -119,10 +114,9 @@ struct ScopedArrayBufferContents: public Scoped<ScopedArrayBufferContentsTraits>
   bool Allocate(uint32_t length) {
     dispose();
     ArrayBufferContents& value = rwget();
-    if (JS_AllocateArrayBufferContents(/*no context available*/nullptr,
-                                       length,
-                                       &value.header,
-                                       &value.data)) {
+    void *ptr = JS_AllocateArrayBufferContents(/*no context available*/nullptr, length);
+    if (ptr) {
+      value.data = (uint8_t *) ptr;
       value.nbytes = length;
       return true;
     }
@@ -360,7 +354,7 @@ nsresult
 TypedArrayResult::GetCacheableResult(JSContext* cx, JS::MutableHandle<JS::Value> aResult)
 {
   MOZ_ASSERT(NS_IsMainThread());
-  // We cannot simply construct a typed array using contents.header as
+  // We cannot simply construct a typed array using contents.data as
   // this would allow us to have several otherwise unrelated
   // ArrayBuffers with the same underlying C buffer. As this would be
   // very unsafe, we need to cache the result once we have it.
@@ -369,7 +363,7 @@ TypedArrayResult::GetCacheableResult(JSContext* cx, JS::MutableHandle<JS::Value>
   MOZ_ASSERT(contents.data);
 
   JS::Rooted<JSObject*>
-    arrayBuffer(cx, JS_NewArrayBufferWithContents(cx, contents.header));
+    arrayBuffer(cx, JS_NewArrayBufferWithContents(cx, contents.nbytes, contents.data));
   if (!arrayBuffer) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -529,7 +523,7 @@ public:
                                                 aDiscardedResult,
                                                 aOperation,
                                                 aOSError);
-    nsresult rv = NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+    nsresult rv = NS_DispatchToMainThread(event);
     if (NS_FAILED(rv)) {
       // Last ditch attempt to release on the main thread - some of
       // the members of event are not thread-safe, so letting the
@@ -547,7 +541,7 @@ public:
     nsRefPtr<SuccessEvent> event = new SuccessEvent(mOnSuccess.forget(),
                                                     mOnError.forget(),
                                                     aResult);
-    nsresult rv = NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+    nsresult rv = NS_DispatchToMainThread(event);
     if (NS_FAILED(rv)) {
       // Last ditch attempt to release on the main thread - some of
       // the members of event are not thread-safe, so letting the
@@ -865,7 +859,7 @@ protected:
 
 // The OS.File service
 
-NS_IMPL_ISUPPORTS1(NativeOSFileInternalsService, nsINativeOSFileInternalsService);
+NS_IMPL_ISUPPORTS(NativeOSFileInternalsService, nsINativeOSFileInternalsService);
 
 NS_IMETHODIMP
 NativeOSFileInternalsService::Read(const nsAString& aPath,
