@@ -28,6 +28,7 @@
 
 #ifdef XP_WIN
 #include <process.h>
+#include "mozilla/ipc/WindowsMessageLoop.h"
 #endif
 
 #include "nsAppDirectoryServiceDefs.h"
@@ -68,6 +69,8 @@
 #include "mozilla/ipc/TestShellParent.h"
 #include "mozilla/ipc/XPCShellEnvironment.h"
 
+#include "GMPProcessChild.h"
+
 #include "GeckoProfiler.h"
 
 #ifdef MOZ_IPDL_TESTS
@@ -300,7 +303,9 @@ XRE_InitChildProcess(int aArgc,
 
   char aLocal;
   profiler_init(&aLocal);
-  PROFILER_LABEL("Startup", "XRE_InitChildProcess");
+
+  PROFILER_LABEL("Startup", "XRE_InitChildProcess",
+    js::ProfileEntry::Category::OTHER);
 
   sChildProcessType = aProcess;
 
@@ -471,6 +476,10 @@ XRE_InitChildProcess(int aArgc,
     {
       nsAutoPtr<ProcessChild> process;
 
+#ifdef XP_WIN
+      mozilla::ipc::windows::InitUIThread();
+#endif
+
       switch (aProcess) {
       case GeckoProcessType_Default:
         NS_RUNTIMEABORT("This makes no sense");
@@ -502,6 +511,10 @@ XRE_InitChildProcess(int aArgc,
 #endif
         break;
 
+      case GeckoProcessType_GMPlugin:
+        process = new gmp::GMPProcessChild(parentHandle);
+        break;
+
       default:
         NS_RUNTIMEABORT("Unknown main thread class");
       }
