@@ -42,8 +42,6 @@
 #include <sys/prctl.h> // set name
 #include <stdlib.h>
 #include <sched.h>
-#include <iostream>
-#include <fstream>
 #ifdef ANDROID
 #include <android/log.h>
 #else
@@ -69,6 +67,7 @@
 #include "GeckoProfiler.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/Atomics.h"
+#include "mozilla/LinuxSignal.h"
 #include "ProfileEntry.h"
 #include "nsThreadUtils.h"
 #include "TableTicker.h"
@@ -79,10 +78,17 @@
 #include "EHABIStackWalk.h"
 #endif
 
+// Memory profile
+#include "nsMemoryReporterManager.h"
+
 #include <string.h>
 #include <stdio.h>
 #include <list>
 
+#ifdef MOZ_NUWA_PROCESS
+#include "ipc/Nuwa.h"
+#endif
+
 #define SIGNAL_SAVE_PROFILE SIGUSR2
 
 #if defined(__GLIBC__)
@@ -133,8 +139,7 @@ static void paf_parent(void) {
     Sampler::GetActiveSampler()->SetPaused(was_paused);
 }
 
-// Set up the fork handlers.  This is called just once, at the first
-// call to SenderEntry.
+// Set up the fork handlers.
 static void* setup_atfork() {
   pthread_atfork(paf_prepare, paf_parent, NULL);
   return NULL;
@@ -204,7 +209,10 @@ static void SetSampleContext(TickSample* sample, void* context)
 #else
 #define V8_HOST_ARCH_X64 1
 #endif
-static void ProfilerSignalHandler(int signal, siginfo_t* info, void* context) {
+
+namespace {
+
+void ProfilerSignalHandler(int signal, siginfo_t* info, void* context) {
   if (!Sampler::GetActiveSampler()) {
     sem_post(&sSignalHandlingDone);
     return;
@@ -222,6 +230,8 @@ static void ProfilerSignalHandler(int signal, siginfo_t* info, void* context) {
 #endif
   sample->threadProfile = sCurrentThreadProfile;
   sample->timestamp = mozilla::TimeStamp::Now();
+  sample->rssMemory = sample->threadProfile->mRssMemory;
+  sample->ussMemory = sample->threadProfile->mUssMemory;
 
   Sampler::GetActiveSampler()->Tick(sample);
 
@@ -229,9 +239,29 @@ static void ProfilerSignalHandler(int signal, siginfo_t* info, void* context) {
   sem_post(&sSignalHandlingDone);
 }
 
+} // namespace
+
+static void ProfilerSignalThread(ThreadProfile *profile,
+                                 bool isFirstProfiledThread)
+{
+  if (isFirstProfiledThread && Sampler::GetActiveSampler()->ProfileMemory()) {
+    profile->mRssMemory = nsMemoryReporterManager::ResidentFast();
+    profile->mUssMemory = nsMemoryReporterManager::ResidentUnique();
+  } else {
+    profile->mRssMemory = 0;
+    profile->mUssMemory = 0;
+  }
+}
+
+// If the Nuwa process is enabled, we need to use the wrapper of tgkill() to
+// perform the mapping of thread ID.
+#ifdef MOZ_NUWA_PROCESS
+extern "C" MFBT_API int tgkill(pid_t tgid, pid_t tid, int signalno);
+#else
 int tgkill(pid_t tgid, pid_t tid, int signalno) {
   return syscall(SYS_tgkill, tgid, tid, signalno);
 }
+#endif
 
 class PlatformData : public Malloced {
  public:
@@ -254,14 +284,18 @@ Sampler::FreePlatformData(PlatformData* aData)
 static void* SignalSender(void* arg) {
   // Taken from platform_thread_posix.cc
   prctl(PR_SET_NAME, "SamplerThread", 0, 0, 0);
-# if defined(ANDROID)
-  // pthread_atfork isn't available on Android.
-  void* initialize_atfork = NULL;
-# else
-  // This call is done just once, at the first call to SenderEntry.
-  // It returns NULL.
-  static void* initialize_atfork = setup_atfork();
-# endif
+
+#ifdef MOZ_NUWA_PROCESS
+  // If the Nuwa process is enabled, we need to mark and freeze the sampler
+  // thread in the Nuwa process and have this thread recreated in the spawned
+  // child.
+  if(IsNuwaProcess()) {
+    NuwaMarkCurrentThread(nullptr, nullptr);
+    // Freeze the thread here so the spawned child will get the correct tgid
+    // from the getpid() call below.
+    NuwaFreezeCurrentThread();
+  }
+#endif
 
   int vm_tgid_ = getpid();
 
@@ -273,6 +307,7 @@ static void* SignalSender(void* arg) {
       std::vector<ThreadInfo*> threads =
         SamplerRegistry::sampler->GetRegisteredThreads();
 
+      bool isFirstProfiledThread = true;
       for (uint32_t i = 0; i < threads.size(); i++) {
         ThreadInfo* info = threads[i];
 
@@ -280,12 +315,28 @@ static void* SignalSender(void* arg) {
         if (!info->Profile())
           continue;
 
+        PseudoStack::SleepState sleeping = info->Stack()->observeSleeping();
+        if (sleeping == PseudoStack::SLEEPING_AGAIN) {
+          info->Profile()->DuplicateLastSample();
+          //XXX: This causes flushes regardless of jank-only mode
+          info->Profile()->flush();
+          continue;
+        }
+
         // We use sCurrentThreadProfile the ThreadProfile for the
         // thread we're profiling to the signal handler
         sCurrentThreadProfile = info->Profile();
 
         int threadId = info->ThreadId();
 
+        // Profile from the signal sender for information which is not signal
+        // safe, and will have low variation between the emission of the signal
+        // and the signal handler catch.
+        ProfilerSignalThread(sCurrentThreadProfile, isFirstProfiledThread);
+
+        // Profile from the signal handler for information which is signal safe
+        // and needs to be precise too, such as the stack of the interrupted
+        // thread.
         if (tgkill(vm_tgid_, threadId, SIGPROF) != 0) {
           printf_stderr("profiler failed to signal tid=%d\n", threadId);
 #ifdef DEBUG
@@ -296,6 +347,7 @@ static void* SignalSender(void* arg) {
 
         // Wait for the signal handler to run before moving on to the next one
         sem_wait(&sSignalHandlingDone);
+        isFirstProfiledThread = false;
       }
     }
 
@@ -308,7 +360,7 @@ static void* SignalSender(void* arg) {
     }
     OS::SleepMicro(interval);
   }
-  return initialize_atfork; // which is guaranteed to be NULL
+  return 0;
 }
 
 Sampler::Sampler(double interval, bool profiling, int entrySize)
@@ -342,7 +394,7 @@ void Sampler::Start() {
   // Request profiling signals.
   LOG("Request signal");
   struct sigaction sa;
-  sa.sa_sigaction = ProfilerSignalHandler;
+  sa.sa_sigaction = MOZ_SIGNAL_TRAMPOLINE(ProfilerSignalHandler);
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = SA_RESTART | SA_SIGINFO;
   if (sigaction(SIGPROF, &sa, &old_sigprof_signal_handler_) != 0) {
@@ -545,7 +597,7 @@ static void StartSignalHandler(int signal, siginfo_t* info, void* context) {
   freeArray(features, featureCount);
 }
 
-void OS::RegisterStartHandler()
+void OS::Startup()
 {
   LOG("Registering start signal");
   struct sigaction sa;
@@ -556,8 +608,18 @@ void OS::RegisterStartHandler()
     LOG("Error installing signal");
   }
 }
+
+#else
+
+void OS::Startup() {
+  // Set up the fork handlers.
+  setup_atfork();
+}
+
 #endif
 
+
+
 void TickSample::PopulateContext(void* aContext)
 {
   MOZ_ASSERT(aContext);
@@ -570,6 +632,24 @@ void TickSample::PopulateContext(void* aContext)
 
 void OS::SleepMicro(int microseconds)
 {
-  usleep(microseconds);
-}
+  if (MOZ_UNLIKELY(microseconds >= 1000000)) {
+    // Use usleep for larger intervals, because the nanosleep
+    // code below only supports intervals < 1 second.
+    MOZ_ALWAYS_TRUE(!::usleep(microseconds));
+    return;
+  }
 
+  struct timespec ts;
+  ts.tv_sec  = 0;
+  ts.tv_nsec = microseconds * 1000UL;
+
+  int rv = ::nanosleep(&ts, &ts);
+
+  while (rv != 0 && errno == EINTR) {
+    // Keep waiting in case of interrupt.
+    // nanosleep puts the remaining time back into ts.
+    rv = ::nanosleep(&ts, &ts);
+  }
+
+  MOZ_ASSERT(!rv, "nanosleep call failed");
+}
